@@ -106,6 +106,10 @@ public:
   */
   void WriteSettings();
 
+private:
+  const int mMinPanelWidth = 640;
+  const int mMinPanelHeight = 720;
+
 private:
   enum newVariables
   {
@@ -280,10 +284,12 @@ protected:
   wxTextCtrl *m_texPreamble;
   wxCheckBox *m_autoSave;
   wxButton *m_mpBrowse;
+  wxButton *m_wxMathMLBrowse;
   wxTextCtrl *m_additionalParameters;
   wxTextCtrl *m_mathJaxURL;
   wxChoice *m_language;
   wxTextCtrl *m_symbolPaneAdditionalChars;
+  wxCheckBox *m_dockableWizards;
   wxCheckBox *m_abortOnError;
   wxCheckBox *m_offerKnownAnswers;
   wxCheckBox *m_restartOnReEvaluation;
@@ -294,7 +300,13 @@ protected:
   wxSpinCtrl *m_defaultPlotWidth;
   wxSpinCtrl *m_defaultPlotHeight;
   wxSpinCtrl *m_displayedDigits;
+  wxRadioButton *m_displayNDigits;
+  wxRadioButton *m_displayAllDigits;
+  wxRadioButton *m_linebreaksInLongNums;
+  wxRadioButton *m_automaticWxMathML;
+  wxRadioButton *m_userWxMathML;
   wxSpinCtrl *m_maxClipbrdBitmapMegabytes;
+  wxTextCtrl *m_wxMathMLLocation;
   //! A checkbox that asks if TeX should put the exponents above or after the subscripts.
   wxCheckBox *m_TeXExponentsAfterSubscript;
   //! A checkbox that asks if TeX should use the \\partial symbol for representing diff()
@@ -304,11 +316,13 @@ protected:
   wxCheckBox *m_printBrackets;
   wxChoice *m_exportWithMathJAX;
   wxCheckBox *m_matchParens;
+  wxCheckBox *m_showMatchingParens;
   wxChoice *m_showLength;
   wxChoice *m_autosubscript;
   wxRadioButton *m_enterEvaluates;
   wxRadioButton *m_ctrlEnterEvaluates;
   wxCheckBox *m_numpadEnterEvaluates;
+  wxCheckBox *m_saveImgFileName;
   wxCheckBox *m_saveUntitled;
   wxCheckBox *m_openHCaret;
   wxCheckBox *m_insertAns;
@@ -347,6 +361,7 @@ protected:
   wxSpinCtrl *m_defaultPort;
   ExamplePanel *m_examplePanel;
   wxSpinCtrl *m_maxGnuplotMegabytes;
+  wxSpinCtrl *m_autosaveMinutes;
   wxTextCtrl *m_autoMathJaxURL;
   int m_maximaEmvRightClickRow = 0;
   //! Is called when the path to the maxima binary was changed.
@@ -361,6 +376,8 @@ protected:
   //! Starts the file chooser that allows selecting where the maxima binary lies
   void OnMpBrowse(wxCommandEvent &event);
 
+  void OnwxMathMLBrowse(wxCommandEvent &event);
+
   void OnHelpBrowserBrowse(wxCommandEvent&  event);
 
   void OnIdle(wxIdleEvent &event);
