@@ -44,6 +44,38 @@
 Configuration::Configuration(wxDC *dc, InitOpt options) :
   m_dc(dc)
 {
+  m_maximaOperators["("] = 1;
+  m_maximaOperators["/"] = 1;
+  m_maximaOperators["{"] = 1;
+  m_maximaOperators["-"] = 1;
+  m_maximaOperators["^"] = 1;
+  m_maximaOperators["#"] = 1;
+  m_maximaOperators["="] = 1;
+  m_maximaOperators[":"] = 1;
+  m_maximaOperators["["] = 1;
+  m_maximaOperators["'"] = 1;
+  m_maximaOperators["!"] = 1;
+  m_maximaOperators["+"] = 1;
+  m_maximaOperators["*"] = 1;
+  m_maximaOperators["or"] = 1;
+  m_maximaOperators["and"] = 1;
+  m_maximaOperators["do_in"] = 1;
+  m_maximaOperators[">"] = 1;
+  m_maximaOperators["$SUBVAR"] = 1;
+  m_maximaOperators["<"] = 1;
+  m_maximaOperators["if"] = 1;
+  m_maximaOperators["::="] = 1;
+  m_maximaOperators["::"] = 1;
+  m_maximaOperators["@"] = 1;
+  m_maximaOperators["."] = 1;
+  m_maximaOperators["-->"] = 1;
+  m_maximaOperators["^^"] = 1;
+  m_maximaOperators["not"] = 1;
+  m_maximaOperators["<="] = 1;
+  m_maximaOperators[":="] = 1;
+  m_maximaOperators[">="] = 1;
+  m_maximaOperators["$BFLOAT"] = 1;
+  m_maximaOperators["do"] = 1;
   m_printing = false;
   m_clipToDrawRegion = true;
   m_inLispMode = false;
@@ -55,11 +87,52 @@ Configuration::Configuration(wxDC *dc, InitOpt options) :
   SetBackgroundBrush(*wxWHITE_BRUSH);
   ResetAllToDefaults(options);
   ReadConfig();
+  wxString operators(
+    wxT("\u221A\u22C0\u22C1\u22BB\u22BC\u22BD\u00AC\u222b\u2264\u2265\u2211\u2260+-*/^:=#'!()[]{}")
+    );
+  for (wxString::const_iterator it = operators.begin(); it != operators.end(); ++it)
+    m_maximaOperators[wxString(*it)] = 1;
+}
+
+wxSize Configuration::GetPPI() const
+{
+  if(GetWorkSheet())
+  {
+    wxSize ppi;
+#if wxCHECK_VERSION(3, 1, 1)
+    wxDisplay display;
+    
+    int display_idx = wxDisplay::GetFromWindow(GetWorkSheet());
+    if(display_idx < 0)
+      ppi = wxSize(72, 72);
+    else
+      ppi = wxDisplay(display_idx).GetPPI();
+#else
+    ppi = wxGetDisplayPPI();
+#endif
+    if((ppi.x < 10) || (ppi.y < 10))
+      ppi = wxGetDisplayPPI();
+    if((ppi.x <= 10) || (ppi.y < 10))
+      ppi = wxSize(72, 72);
+    
+    return ppi;;
+  }
+  else
+    return m_ppi;
 }
 
 void Configuration::ResetAllToDefaults(InitOpt options)
 {
+  m_dockableWizards = true;
+  for(auto i:m_renderableChars)
+    m_renderableChars[i.first] = wxEmptyString;
+  for(auto i:m_nonRenderableChars)
+    m_nonRenderableChars[i.first] = wxEmptyString;
+  m_showAllDigits = false;
+  m_lineBreaksInLongNums = false;
+  m_autoSaveMinutes = 3;
   m_numpadEnterEvaluates = true;
+  m_saveImgFileName = false;
   m_maximaEnvVars.clear();
   // Tell gnuplot not to wait for <enter> every few lines
   #ifndef __WXMSW__
@@ -70,6 +143,7 @@ void Configuration::ResetAllToDefaults(InitOpt options)
   m_bitmapScale = 3;
   m_maxClipbrd_BitmapMegabytes = 4;
   m_defaultFramerate = 12;
+  m_tocDepth = 255;
   m_fixedFontTC = false;
   m_hideMarkerForThisMessage.clear();
   #ifdef __WXOSX__
@@ -77,6 +151,9 @@ void Configuration::ResetAllToDefaults(InitOpt options)
   #else
   m_usepngCairo = true;
   #endif
+  m_wxMathML_Filename = wxEmptyString;
+  m_wxMathML_UseFile = false;
+
   m_mathJaxURL = wxT("https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js");
   m_usePartialForDiff = false,
   m_documentclass = wxT("article");
@@ -159,6 +236,7 @@ void Configuration::ResetAllToDefaults(InitOpt options)
   m_autoIndent = true;
   m_restartOnReEvaluation = true;
   m_matchParens = true;
+  m_showMatchingParens = true;
   m_insertAns = false;
   m_openHCaret = false;
   m_labelWidth = 4;
@@ -375,29 +453,6 @@ Configuration::EscCodeIterator Configuration::EscCodesBegin()
 Configuration::EscCodeIterator Configuration::EscCodesEnd()
 { return EscCodes().cend(); }
 
-wxSize Configuration::GetPPI(wxWindow *win) const
-{
-  if(win == NULL)
-    return wxSize(96,96);
-  
-  wxSize ppi(-1,-1);
-#if wxCHECK_VERSION(3, 1, 1)
-  wxDisplay display;
-  
-  int display_idx = wxDisplay::GetFromWindow(win);
-  if (display_idx < 0)
-    ppi = wxSize(72,72);
-  else
-    ppi = wxDisplay(display_idx).GetPPI();
-#endif
-
-  if((ppi.x < 10) || (ppi.y < 10))
-    ppi = wxGetDisplayPPI();
-  if((ppi.x <= 10) || (ppi.y <= 10))
-    ppi = wxSize(72,72);
-
-  return ppi;
-}
 
 wxString Configuration::GetAutosubscript_string() const
 {
@@ -455,10 +510,50 @@ void Configuration::ReadConfig()
 {
   wxConfigBase *config = wxConfig::Get();
 
+  wxString str;
+  long dummy;
+  config->SetPath("/renderability/good");
+  wxLogMessage(_("Reading the glyph renderability cache"));
+  bool bCont = config->GetFirstEntry(str, dummy);
+  while (bCont)
+  {
+    wxString chars;
+    config->Read(str, &chars);
+    wxLogMessage(
+      wxString::Format(
+        _("Font %s should render the glyphs \"%s\" correctly."),
+        str.c_str(),
+        chars.c_str()
+        )
+      );
+    m_renderableChars[str] = chars;
+    bCont = config->GetNextEntry(str, dummy);
+  }
+  config->SetPath("/renderability/bad");
+  bCont = config->GetFirstEntry(str, dummy);
+  while (bCont)
+  {
+    wxString chars;
+    config->Read(str, &chars);
+    wxLogMessage(
+      wxString::Format(
+        _("Font %s cannot render the glyphs \"%s\"."),
+        str.c_str(),
+        chars.c_str()
+        )
+      );
+    m_nonRenderableChars[str] = chars;
+    bCont = config->GetNextEntry(str, dummy);
+  }
+  config->SetPath("/");
+
   {
     // If this preference cannot be loaded we don't want an error message about it
     SuppressErrorDialogs suppressor;
     wxString hideMessagesConfigString;
+    config->Read(wxT("showAllDigits"), &m_showAllDigits);
+    config->Read(wxT("lineBreaksInLongNums"), &m_lineBreaksInLongNums);
+    config->Read(wxT("autoSaveMinutes"), &m_autoSaveMinutes);
     config->Read(wxT("wrapLatexMath"), &m_wrapLatexMath);
     config->Read(wxT("exportContainsWXMX"), &m_exportContainsWXMX);
     config->Read(wxT("texPreamble"), &m_texPreamble);
@@ -565,12 +660,15 @@ void Configuration::ReadConfig()
   if(m_maxClipbrd_BitmapMegabytes<0)
     m_maxClipbrd_BitmapMegabytes = 1;
   config->Read(wxT("numpadEnterEvaluates"), &m_numpadEnterEvaluates);
+  config->Read(wxT("saveImgFileName"), &m_saveImgFileName);
   config->Read(wxT("usePartialForDiff"), &m_usePartialForDiff);
   config->Read(wxT("TeXExponentsAfterSubscript"), &m_TeXExponentsAfterSubscript);
   config->Read(wxT("defaultPlotWidth"), &m_defaultPlotWidth);
   config->Read(wxT("defaultPlotHeight"), &m_defaultPlotHeight);
   config->Read(wxT("fixedFontTC"), &m_fixedFontTC);
   config->Read(wxT("usepngCairo"), &m_usepngCairo);
+  config->Read(wxT("dockableWizards"), &m_dockableWizards);
+  
   if(!config->Read(wxT("AutoSaveAsTempFile"), &m_autoSaveAsTempFile))
   {
     long autoSaveMinutes = 0;
@@ -582,6 +680,8 @@ void Configuration::ReadConfig()
   if (m_language == wxLANGUAGE_UNKNOWN)
     m_language = wxLANGUAGE_DEFAULT;
 
+  config->Read("wxMathML_Filename", &m_wxMathML_Filename);
+  config->Read("wxMathML_UseFile", &m_wxMathML_UseFile);
   config->Read("invertBackground", &m_invertBackground);
   config->Read("undoLimit", &m_undoLimit);
   config->Read("recentItems", &m_recentItems);
@@ -621,6 +721,9 @@ void Configuration::ReadConfig()
   config->Read(wxT("copyBitmap"), &m_copyBitmap);
   config->Read(wxT("bitmapScale"), &m_bitmapScale);
   config->Read(wxT("DefaultFramerate"), &m_defaultFramerate);
+  config->Read(wxT("tocDepth"), &m_tocDepth);
+  if(m_tocDepth < 1)
+    m_tocDepth = 1;
   config->Read(wxT("copyBitmap"), &m_copyBitmap);
   config->Read(wxT("copyMathML"), &m_copyMathML);
   config->Read(wxT("copyMathMLHTML"), &m_copyMathMLHTML);
@@ -652,6 +755,7 @@ void Configuration::ReadConfig()
   config->Read(wxT("restartOnReEvaluation"), &m_restartOnReEvaluation);
 
   config->Read(wxT("matchParens"), &m_matchParens);
+  config->Read(wxT("showMatchingParens"), &m_showMatchingParens);
 
   config->Read(wxT("insertAns"), &m_insertAns);
 
@@ -1042,7 +1146,7 @@ void Configuration::ReadStyles(const wxString &file)
   m_BackgroundBrush = *wxTheBrushList->FindOrCreateBrush(m_styles[TS_DOCUMENT_BACKGROUND].GetColor(), wxBRUSHSTYLE_SOLID);
 }
 
-//! Saves the style settings to a file.
+//! Saves the settings to a file.
 void Configuration::WriteSettings(const wxString &file)
 {
   wxConfigBase *config = NULL;
@@ -1153,10 +1257,16 @@ void Configuration::WriteSettings(const wxString &file)
     config->Write(wxT("suppressYellowMarkerMessages"), hideMessagesConfigString);
   }
   
+  config->Write(wxT("showAllDigits"), m_showAllDigits);
+  config->Write(wxT("lineBreaksInLongNums"), m_lineBreaksInLongNums);
   config->Write(wxT("keepPercent"), m_keepPercent);
   config->Write(wxT("labelWidth"), m_labelWidth);
   config->Write(wxT("saveUntitled"), m_saveUntitled);
   config->Write(wxT("cursorJump"), m_cursorJump);
+  config->Write(wxT("autoSaveMinutes"), m_autoSaveMinutes);
+  config->Write(wxT("wxMathML_Filename"), m_wxMathML_Filename);
+  config->Write(wxT("wxMathML_UseFile"), m_wxMathML_UseFile);
+  config->Write(wxT("dockableWizards"), m_dockableWizards);
 
   config->Write(wxT("maxClipbrd_BitmapMegabytes"), m_maxClipbrd_BitmapMegabytes);
 
@@ -1166,6 +1276,10 @@ void Configuration::WriteSettings(const wxString &file)
     config->Flush();
     delete config;
   }
+  for(auto i:m_renderableChars)
+    config->Write(wxT("renderability/good/")+i.first, i.second);
+  for(auto i:m_nonRenderableChars)
+    config->Write(wxT("renderability/bad/")+i.first, i.second);
 }
 
 wxFontWeight Configuration::IsBold(long st) const
@@ -1247,12 +1361,139 @@ bool Configuration::InUpdateRegion(wxRect const rect) const
          rect.Contains(updateRegion);
 }
 
+bool Configuration::FontRendersChar(wxChar ch, const wxFont &font)
+{
+  wxString fontName = font.GetNativeFontInfoDesc();
+  fontName.Replace("/","_");
+  if(m_renderableChars[fontName].Contains(ch))
+    return true;
+  if(m_nonRenderableChars[fontName].Contains(ch))
+    return false;
+  
+  bool retval = 
+    FontDisplaysChar(ch, font) &&
+    CharVisiblyDifferent(ch, wxT('\1'), font) &&
+    CharVisiblyDifferent(ch, wxT('\uF299'), font) &&
+    CharVisiblyDifferent(ch, wxT('\uF000'), font);
+
+  if(retval)
+    m_renderableChars[fontName] += wxString(ch);
+  else
+    m_nonRenderableChars[fontName] += wxString(ch);
+    
+  return retval;
+}
+
+bool Configuration::FontDisplaysChar(wxChar ch, const wxFont &font)
+{
+  int width = 200;
+  int height = 200;
+
+  // Prepare two identical device contexts that create identical bitmaps
+  wxBitmap characterBitmap = wxBitmap(wxSize(width,height),
+                                      wxBITMAP_SCREEN_DEPTH
+    );
+  wxBitmap referenceBitmap = wxBitmap(wxSize(width,height),
+                                      wxBITMAP_SCREEN_DEPTH
+    );
+  wxMemoryDC characterDC;
+  wxMemoryDC referenceDC;
+  characterDC.SetFont(font);
+  referenceDC.SetFont(font);
+  characterDC.SelectObject(characterBitmap);
+  referenceDC.SelectObject(referenceBitmap);
+  characterDC.SetBrush(*wxWHITE_BRUSH);
+  referenceDC.SetBrush(*wxWHITE_BRUSH);
+  characterDC.DrawRectangle(0,0,200,200);
+  referenceDC.DrawRectangle(0,0,200,200);
+  characterDC.SetPen(*wxBLACK_PEN);
+  referenceDC.SetPen(*wxBLACK_PEN);
+
+  // Now draw the character our button shows into one of these bitmaps and see
+  // if that changed any aspect of the bitmap
+  characterDC.DrawText(wxString(ch),
+                       100,
+                       100);
+  wxImage characterImage = characterBitmap.ConvertToImage(); 
+  wxImage referenceImage = referenceBitmap.ConvertToImage(); 
+  for(int x=0; x < width; x++)
+    for(int y=0; y < height; y++)
+    {
+      if(characterImage.GetRed(x,y) != referenceImage.GetRed(x,y))
+        return true;
+      if(characterImage.GetGreen(x,y) != referenceImage.GetGreen(x,y))
+        return true;
+      if(characterImage.GetBlue(x,y) != referenceImage.GetBlue(x,y))
+        return true;
+    }
+  wxLogMessage(
+    wxString::Format(
+      wxT("Char '%s' seems not to be displayed."),
+      wxString(ch).c_str()));
+
+  // characterImage.SaveFile(wxString(m_char)+".png");
+  
+  return false;
+}
+
+bool Configuration::CharVisiblyDifferent(wxChar ch, wxChar otherChar, const wxFont &font)
+{
+  int width = 200;
+  int height = 200;
+
+  // Prepare two identical device contexts that create identical bitmaps
+  wxBitmap characterBitmap = wxBitmap(wxSize(width,height),
+                                      wxBITMAP_SCREEN_DEPTH
+    );
+  wxBitmap referenceBitmap = wxBitmap(wxSize(width,height),
+                                      wxBITMAP_SCREEN_DEPTH
+    );
+  wxMemoryDC characterDC;
+  wxMemoryDC referenceDC;
+  characterDC.SetFont(font);
+  referenceDC.SetFont(font);
+  characterDC.SelectObject(characterBitmap);
+  referenceDC.SelectObject(referenceBitmap);
+  characterDC.SetBrush(*wxWHITE_BRUSH);
+  referenceDC.SetBrush(*wxWHITE_BRUSH);
+  characterDC.DrawRectangle(0,0,200,200);
+  referenceDC.DrawRectangle(0,0,200,200);
+  characterDC.SetPen(*wxBLACK_PEN);
+  referenceDC.SetPen(*wxBLACK_PEN);
+  characterDC.DrawText(wxString(ch),
+                       100,
+                       100);
+  referenceDC.DrawText(wxString(otherChar),
+                       100,
+                       100);
+  wxImage characterImage = characterBitmap.ConvertToImage(); 
+  wxImage referenceImage = referenceBitmap.ConvertToImage(); 
+  for(int x=0; x < width; x++)
+    for(int y=0; y < height; y++)
+    {
+      if(characterImage.GetRed(x,y) != referenceImage.GetRed(x,y))
+        return true;
+      if(characterImage.GetGreen(x,y) != referenceImage.GetGreen(x,y))
+        return true;
+      if(characterImage.GetBlue(x,y) != referenceImage.GetBlue(x,y))
+        return true;
+    }
+  wxLogMessage(
+    wxString::Format(
+      wxT("Char '%s' looks identical to '%s'."),
+      wxString(ch).c_str(),
+      wxString(otherChar).c_str()));
+  return false;
+}
+
+
 void Configuration::WriteStyles(wxConfigBase *config)
 {
   config->Write(wxT("wrapLatexMath"), m_wrapLatexMath);
   config->Write(wxT("exportContainsWXMX"), m_exportContainsWXMX);
   config->Write(wxT("texPreamble"), m_texPreamble);
   config->Write(wxT("numpadEnterEvaluates"), m_numpadEnterEvaluates);
+  config->Write(wxT("saveImgFileName"), m_saveImgFileName);
   config->Write(wxT("usePartialForDiff"), m_usePartialForDiff);
   config->Write(wxT("TeXExponentsAfterSubscript"), m_TeXExponentsAfterSubscript);
   config->Write(wxT("defaultPlotWidth"), m_defaultPlotWidth);
@@ -1260,6 +1501,7 @@ void Configuration::WriteStyles(wxConfigBase *config)
   config->Write(wxT("fixedFontTC"), m_fixedFontTC);
   config->Write(wxT("bitmapScale"), m_bitmapScale);
   config->Write(wxT("DefaultFramerate"), m_defaultFramerate);
+  config->Write(wxT("tocDepth"), m_tocDepth);
   config->Write(wxT("usepngCairo"), m_usepngCairo);
   config->Write("incrementalSearch", m_incrementalSearch);
   config->Write(wxT("hideBrackets"), m_hideBrackets);
@@ -1269,6 +1511,7 @@ void Configuration::WriteStyles(wxConfigBase *config)
   config->Write(wxT("autoIndent"), m_autoIndent);
   config->Write(wxT("indentMaths"), m_indentMaths);
   config->Write(wxT("matchParens"), m_matchParens);
+  config->Write(wxT("showMatchingParens"), m_showMatchingParens);
   config->Write(wxT("changeAsterisk"), m_changeAsterisk);
   config->Write(wxT("hidemultiplicationsign"), m_hidemultiplicationsign);
   config->Write(wxT("latin2greek"), m_latin2greek);
