@@ -123,6 +123,11 @@ public:
     display_1dASCII
   };
 
+  WX_DECLARE_STRING_HASH_MAP(int, StringHash);
+  /*! All maxima operator names we know
+  */
+  StringHash m_maximaOperators;
+
   const wxEnvVariableHashMap& MaximaEnvVars(){return m_maximaEnvVars;}
   wxEnvVariableHashMap m_maximaEnvVars;
 
@@ -279,14 +284,16 @@ public:
       return m_indent;
   }
 
-  //! Get the resolution of the display showing the worksheet
-  wxSize GetPPI() const {return GetPPI(GetWorkSheet());}
+  void SetPPI(wxSize ppi){m_ppi = ppi;}
+  /*! Get the resolution 
+
+    During recalculation the drawing context doesn't know the display's resolution
+    so we need to do some additional tricks here.
+  */
+  wxSize GetPPI() const;
+  int AutosaveMinutes(){return m_autoSaveMinutes;}
+  void AutosaveMinutes(int minutes){m_autoSaveMinutes = minutes;}
 
-  // cppcheck-suppress functionStatic
-  // cppcheck-suppress functionConst
-  //! Get the resolution of an arbitrary display
-  wxSize GetPPI(wxWindow *win) const;
-  
   //! How much vertical space is to be left between two group cells?
   long GetCursorWidth() const
   {
@@ -375,7 +382,7 @@ public:
       return GetZoomFactor();
   }
 
-  //! The minimum sensible line width in withs of a letter.
+  //! The minimum sensible line width in widths of a letter.
   long LineWidth_em() const 
   {
     if(!m_printing)
@@ -405,6 +412,9 @@ public:
   bool NumpadEnterEvaluates(){ return m_numpadEnterEvaluates;}
   void NumpadEnterEvaluates(bool eval){m_numpadEnterEvaluates = eval;}
 
+  bool SaveImgFileName() { return m_saveImgFileName;}
+  void SaveImgFileName(bool save) { m_saveImgFileName = save;}
+
   //! Do we want to have automatic line breaks for text cells?
   bool GetAutoWrap() const
   { return m_autoWrap > 0; }
@@ -524,6 +534,8 @@ public:
 
   bool GetMatchParens() const { return m_matchParens; }
   void SetMatchParens(bool matchParens) { m_matchParens = matchParens; }
+  bool ShowMatchingParens() const { return m_showMatchingParens; }
+  void ShowMatchingParens(bool show) { m_showMatchingParens = show; }
   
   bool GetChangeAsterisk() const{ return m_changeAsterisk; }
   
@@ -696,6 +708,10 @@ public:
   bool MathJaXURL_UseUser() const { return m_mathJaxURL_UseUser;}
   void MathJaXURL_UseUser(bool useUser){m_mathJaxURL_UseUser = useUser;}
 
+  bool WxMathML_UseFile() const { return m_wxMathML_UseFile;}
+  void WxMathML_UseFile(bool useFile){m_wxMathML_UseFile = useFile;}
+  wxString WxMathML_Filename() const { return m_wxMathML_Filename;}
+  void WxMathML_Filename(wxString name) { m_wxMathML_Filename = name;}
   bool EnterEvaluates() const {return m_enterEvaluates;}
   void EnterEvaluates(bool enterEvaluates) {m_enterEvaluates = enterEvaluates;}
   static wxString MathJaXURL_Auto() { return wxT("https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js");}
@@ -819,6 +835,8 @@ wxString DocumentclassOptions() const {return m_documentclassOptions;}
   int DefaultPlotWidth() const {return m_defaultPlotWidth;}
   void DefaultFramerate(int fps){m_defaultFramerate = fps;}
   int DefaultFramerate() const {return m_defaultFramerate;}
+  void TocDepth(int depth){m_tocDepth = depth;}
+  int TocDepth() const {return m_tocDepth;}
   bool TeXExponentsAfterSubscript() const {return m_TeXExponentsAfterSubscript;}
   void TeXExponentsAfterSubscript(bool ExponentsAfterSubscript)
     {m_TeXExponentsAfterSubscript = ExponentsAfterSubscript;}
@@ -831,21 +849,40 @@ wxString DocumentclassOptions() const {return m_documentclassOptions;}
 
   bool WrapLatexMath() const {return m_wrapLatexMath;}
   void WrapLatexMath(bool wrapLatexMath){m_wrapLatexMath = wrapLatexMath;}
+  bool ShowAllDigits() const {return m_showAllDigits;}
+  void ShowAllDigits(bool shw){m_showAllDigits = shw;}
+  bool LineBreaksInLongNums() const {return m_lineBreaksInLongNums;}
+  void LineBreaksInLongNums(bool brk){m_lineBreaksInLongNums = brk;}
   int  MaxClipbrdBitmapMegabytes() const {return m_maxClipbrd_BitmapMegabytes;}
   void MaxClipbrdBitmapMegabytes(int maxClipbrd_BitmapMegabytes)
     {m_maxClipbrd_BitmapMegabytes = maxClipbrd_BitmapMegabytes;}
   
   void ExportContainsWXMX(bool exportContainsWXMX){m_exportContainsWXMX = exportContainsWXMX;}
   bool ExportContainsWXMX() const {return m_exportContainsWXMX;}
+  void DockableWizards(bool dockableWizards){m_dockableWizards = dockableWizards;}
+  bool DockableWizards() const {return m_dockableWizards;}
   wxString TexPreamble() const {return m_texPreamble;}
   void TexPreamble(wxString texPreamble) {m_texPreamble = texPreamble;}
 
   Style m_styles[NUMBEROFSTYLES];
   //! Initialize the text styles on construction.
   void InitStyles();
+  //! True if we are confident that the font renders this char
+  bool FontRendersChar(wxChar ch, const wxFont &font = *wxNORMAL_FONT);
+  
 private:
+  WX_DECLARE_STRING_HASH_MAP(wxString, RenderablecharsHash);
+  RenderablecharsHash m_renderableChars;
+  RenderablecharsHash m_nonRenderableChars;
+  //! True if drawing the char this button displays alters at least one pixel
+  bool FontDisplaysChar(wxChar ch, const wxFont &font = *wxNORMAL_FONT);
+  //! True if drawing the char this button displays differs visibly from otherChar
+  bool CharVisiblyDifferent(wxChar ch, wxChar otherChar, const wxFont &font = *wxNORMAL_FONT);
+  //! The ppi rate if we don't have a worksheet that provides a current ppi rate
+  wxSize m_ppi = wxSize(-1, -1);
   mathDisplayMode m_displayMode = display_2d;
   using CellRedrawTrace = std::vector<const Cell*>;
+  bool m_dockableWizards;
   bool m_usePartialForDiff;
   //! true = Autosave doesn't save into the current file.
   bool m_autoSaveAsTempFile;
@@ -862,7 +899,7 @@ private:
   //! Which objects do we want to convert into subscripts if they occur after an underscore?
   long m_autoSubscript;
   //! The worksheet this configuration storage is valid for
-  wxWindow *m_workSheet;
+  wxWindow *m_workSheet = NULL;
   /*! Do these chars exist in the given font?
 
     wxWidgets currently doesn't define such a function. But we can do the following:
@@ -910,6 +947,10 @@ private:
   long m_autoWrap;
   //! Automatically indent long lines?
   bool m_autoIndent;
+  //! Always show all digits of all numbers?
+  bool m_showAllDigits;
+  //! Allow linebreaks in numbers that are longer than a line?
+  bool m_lineBreaksInLongNums;  
   //! Do we want to automatically close parenthesis?
   bool m_matchParens;
   //! Do we want to automatically insert new cells containing a "%" at the end of every command?
@@ -959,6 +1000,7 @@ private:
   bool m_useUnicodeMaths;
   bool m_indentMaths;
   bool m_abortOnError;
+  bool m_showMatchingParens;
   bool m_hidemultiplicationsign;
   bool m_offerKnownAnswers;
   long m_defaultPort;
@@ -968,6 +1010,7 @@ private:
   bool m_saveUntitled;
   bool m_cursorJump;
   bool m_numpadEnterEvaluates;
+  bool m_saveImgFileName;
   std::unique_ptr<CellRedrawTrace> m_cellRedrawTrace;
   wxString m_documentclass;
   wxString m_documentclassOptions;
@@ -991,7 +1034,11 @@ private:
   wxString m_lispType;
   int m_bitmapScale;
   int m_defaultFramerate;
+  int m_tocDepth;
   int m_maxClipbrd_BitmapMegabytes;
+  int m_autoSaveMinutes;
+  wxString m_wxMathML_Filename;
+  bool m_wxMathML_UseFile;
 };
 
 //! Sets the configuration's "printing" flag until this class is left.
