@@ -34,7 +34,7 @@ that still have to be sent to maxima.
 
 #include "precomp.h"
 #include "GroupCell.h"
-#include "wx/arrstr.h"
+#include <wx/arrstr.h>
 #include <vector>
 
 //! A simple FIFO queue with manual removal of elements
@@ -121,7 +121,7 @@ public:
   {};
 
   //! Is GroupCell gr part of the evaluation queue?
-  bool IsLastInQueue(GroupCell *gr)
+  bool IsLastInQueue(GroupCell const *gr)
   {
     return !m_queue.empty() && (gr == m_queue.front());
   }
