@@ -24,9 +24,27 @@
 */
 
 #include "Image.h"
-#define NANOSVG_ALL_COLOR_KEYWORDS
+
+/* The nanosvg .h files contain both the header and the implementation.
+   In exactly one file of the project need to be defined in order to
+   make the implementation visible to the compiler.
+   
+   In wxWidgets >3.1.6 this is done in wxWidgets itself so we need to 
+   skip that step there.
+
+   So that should work. But for Linux builds with 3.1.6 I get linking errors, e.g.:
+   undefined reference to `nsvgRasterize'
+   Therefore do not define NANOSVG_IMPLEMENTATION/NANOSVGRAST_IMPLEMENTATION only for Windows.
+   That is probably not correct (a FIXME), but I have no idea, why Linux still requires
+   that NANOSVG_IMPLEMENTATION/NANOSVGRAST_IMPLEMENTATION is defined.
+
+ */
+#if (wxCHECK_VERSION(3, 1, 6)) && defined(__WINDOWS__)
+#else
 #define NANOSVG_IMPLEMENTATION
 #define NANOSVGRAST_IMPLEMENTATION
+#endif
+
 #include <wx/mstream.h>
 #include <wx/wfstream.h>
 #include <wx/zstream.h>
@@ -39,10 +57,6 @@
 
 Image::Image(Configuration **config)
 {
-  #ifdef HAVE_OMP_HEADER
-  omp_init_lock(&m_gnuplotLock);
-  omp_init_lock(&m_imageLoadLock);
-  #endif
   m_configuration = config;
   m_width = 1;
   m_height = 1;
@@ -56,10 +70,6 @@ Image::Image(Configuration **config)
 
 Image::Image(Configuration **config, wxMemoryBuffer image, wxString type)
 {
-  #ifdef HAVE_OMP_HEADER
-  omp_init_lock(&m_gnuplotLock);
-  omp_init_lock(&m_imageLoadLock);
-  #endif
   m_configuration = config;
   m_scaledBitmap.Create(1, 1);
   m_compressedImage = image;
@@ -87,10 +97,6 @@ Image::Image(Configuration **config, wxMemoryBuffer image, wxString type)
 
 Image::Image(Configuration **config, const wxBitmap &bitmap)
 {
-  #ifdef HAVE_OMP_HEADER
-  omp_init_lock(&m_gnuplotLock);
-  omp_init_lock(&m_imageLoadLock);
-  #endif
   m_configuration = config;
   m_isOk = false;
   m_width = 1;
@@ -108,12 +114,8 @@ Image::Image(Configuration **config, const wxBitmap &bitmap)
 // pointer to the file system alive in a background task
 // cppcheck-suppress performance symbolName=filesystem
 Image::Image(Configuration **config, wxString image, std::shared_ptr<wxFileSystem> filesystem, bool remove):
-    m_fs_keepalive_imagedata(filesystem)
+  m_fs_keepalive_imagedata(filesystem)
 {
-  #ifdef HAVE_OMP_HEADER
-  omp_init_lock(&m_gnuplotLock);
-  omp_init_lock(&m_imageLoadLock);
-  #endif
   m_svgImage = NULL;
   m_configuration = config;
   m_scaledBitmap.Create(1, 1);
@@ -124,35 +126,49 @@ Image::Image(Configuration **config, wxString image, std::shared_ptr<wxFileSyste
   m_maxHeight = -1;
   m_originalWidth = 640;
   m_originalHeight = 480;
+  m_ppi = (*m_configuration)->GetDC()->GetPPI().x;
   LoadImage(image, filesystem, remove);
 }
 
+Image::Image(Configuration **config, const Image &image)
+{
+  m_svgImage = NULL;
+  m_configuration = config;
+  m_scaledBitmap.Create(1, 1);
+  m_isOk = image.m_isOk;
+  m_width = 1;
+  m_height = 1;
+  m_maxWidth = image.m_maxWidth;
+  m_maxHeight = image.m_maxHeight;
+  m_originalWidth = image.m_originalWidth;
+  m_originalHeight = image.m_originalHeight;
+  m_compressedImage = image.m_compressedImage;
+  m_ppi = image.m_ppi;
+  m_extension = image.m_extension;
+}
+
+
 Image::~Image()
 {
   m_isOk = false;
-  #ifdef HAVE_OPENMP_TASKS
-  #pragma omp taskwait
-  #endif
+  if(!m_gnuplotSource.IsEmpty())
   {
-    if(!m_gnuplotSource.IsEmpty())
-    {
-      SuppressErrorDialogs logNull;
-      wxLogMessage(wxString::Format(_("Trying to delete gnuplot file %s"), m_gnuplotSource.utf8_str()));
-      if(wxFileExists(m_gnuplotSource))
-        wxRemoveFile(m_gnuplotSource);
-      wxString popoutname = m_gnuplotSource + wxT(".popout");
-      wxLogMessage(wxString::Format(_("Trying to delete gnuplot file %s"), popoutname.utf8_str()));
-      if(wxFileExists(popoutname))
-        wxRemoveFile(popoutname);    
-    }
+    SuppressErrorDialogs logNull;
+    wxLogMessage(wxString::Format(_("Trying to delete gnuplot file %s"), m_gnuplotSource.utf8_str()));
+    if(wxFileExists(m_gnuplotSource))
+      wxRemoveFile(m_gnuplotSource);
+    wxString popoutname = m_gnuplotSource + wxT(".popout");
+    wxLogMessage(wxString::Format(_("Trying to delete gnuplot file %s"), popoutname.utf8_str()));
+    if(wxFileExists(popoutname))
+      wxRemoveFile(popoutname);    
+  }
 
-    if(!m_gnuplotData.IsEmpty())
-    {
-      SuppressErrorDialogs logNull;
-      wxLogMessage(wxString::Format(_("Trying to delete gnuplot file %s"), m_gnuplotData.utf8_str()));
-      if(wxFileExists(m_gnuplotData))
-        wxRemoveFile(m_gnuplotData);
-    }
+  if(!m_gnuplotData.IsEmpty())
+  {
+    SuppressErrorDialogs logNull;
+    wxLogMessage(wxString::Format(_("Trying to delete gnuplot file %s"), m_gnuplotData.utf8_str()));
+    if(wxFileExists(m_gnuplotData))
+      wxRemoveFile(m_gnuplotData);
   }
   if(m_svgImage)
     free(m_svgImage);
@@ -174,9 +190,6 @@ wxMemoryBuffer Image::ReadCompressedImage(wxInputStream *data)
 
 wxBitmap Image::GetUnscaledBitmap()
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
 
   if(!m_isOk)
   {
@@ -184,6 +197,7 @@ wxBitmap Image::GetUnscaledBitmap()
     return m_scaledBitmap;    
   }
 
+  SuppressErrorDialogs logNull;
   if (m_svgRast)
   {
     std::vector<unsigned char> imgdata(m_originalWidth*m_originalHeight*4);
@@ -205,33 +219,21 @@ wxBitmap Image::GetUnscaledBitmap()
 
 wxMemoryBuffer Image::GetCompressedImage()
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
   return m_compressedImage;
 }
 
 size_t Image::GetOriginalWidth()
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
   return m_originalWidth;
 }
 
 size_t Image::GetOriginalHeight()
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
   return m_originalHeight;
 }
 
 bool Image::IsOk()
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
   return m_isOk;
 }
 
@@ -242,18 +244,11 @@ bool Image::IsOk()
 void Image::GnuplotSource(wxString gnuplotFilename, wxString dataFilename, std::shared_ptr<wxFileSystem> filesystem)
 {
   m_fs_keepalive_gnuplotdata = filesystem;
-  #ifdef HAVE_OPENMP_TASKS
-  wxLogMessage(_("Scheduling background task that loads the gnuplot data for a plot."));
-  #pragma omp task
-  #endif
   LoadGnuplotSource_Backgroundtask(gnuplotFilename, dataFilename, filesystem);
 }
 
 void Image::LoadGnuplotSource_Backgroundtask(wxString gnuplotFilename, wxString dataFilename, std::shared_ptr<wxFileSystem> filesystem)
 {
-  #ifdef HAVE_OMP_HEADER
-  omp_set_lock(&m_gnuplotLock);
-  #endif
   // Error dialogues need to be created by the foreground thread.
   SuppressErrorDialogs suppressor;
 
@@ -271,9 +266,6 @@ void Image::LoadGnuplotSource_Backgroundtask(wxString gnuplotFilename, wxString
       {
         wxLogMessage(_("Too much gnuplot data => Not storing it in the worksheet"));
         m_gnuplotData_Compressed.Clear();
-        #ifdef HAVE_OMP_HEADER
-        omp_unset_lock(&m_gnuplotLock);
-        #endif
         return;
       }
       
@@ -356,9 +348,6 @@ void Image::LoadGnuplotSource_Backgroundtask(wxString gnuplotFilename, wxString
   {
     {
       wxFSFile *fsfile;
-      #ifdef HAVE_OPENMP_TASKS
-      #pragma omp critical (OpenFSFile)
-      #endif
       fsfile = filesystem->OpenFile(m_gnuplotSource);
       if (fsfile)
       { // open successful
@@ -410,9 +399,6 @@ void Image::LoadGnuplotSource_Backgroundtask(wxString gnuplotFilename, wxString
     }
     {
       wxFSFile *fsfile;
-      #ifdef HAVE_OPENMP_TASKS
-      #pragma omp critical (OpenFSFile)
-      #endif
       fsfile = filesystem->OpenFile(m_gnuplotData);
       if (fsfile)
       { // open successful
@@ -453,57 +439,38 @@ void Image::LoadGnuplotSource_Backgroundtask(wxString gnuplotFilename, wxString
     }
   }
   m_fs_keepalive_gnuplotdata.reset();
-  #ifdef HAVE_OMP_HEADER
-  omp_unset_lock(&m_gnuplotLock);
-  #endif
 }
-
 wxMemoryBuffer Image::GetGnuplotSource()
 {
   wxMemoryBuffer retval;
-  #ifdef HAVE_OMP_HEADER
-  omp_set_lock(&m_gnuplotLock);
-  #else
-  #ifdef HAVE_OPENMP_TASKS
-  #pragma omp taskwait
-  #endif
-  #endif
-  {  
-    if(
-      (m_gnuplotSource_Compressed.GetDataLen() < 2) || 
-      (m_gnuplotData_Compressed.GetDataLen() < 2))
+  if(
+    (m_gnuplotSource_Compressed.GetDataLen() < 2) || 
+    (m_gnuplotData_Compressed.GetDataLen() < 2))
+  {
+    return retval;
+  }
+  wxMemoryOutputStream output;
+  wxTextOutputStream textOut(output);
+  if(output.IsOk())
+  {
+    wxMemoryInputStream mstream(
+      m_gnuplotSource_Compressed.GetData(),
+      m_gnuplotSource_Compressed.GetDataLen()
+      );
+    wxZlibInputStream zstream(mstream);
+    wxTextInputStream textIn(zstream);
+    wxString line;
+  
+    while(!zstream.Eof())
     {
-      #ifdef HAVE_OMP_HEADER
-      omp_unset_lock(&m_gnuplotLock);
-      #endif
-      return retval;
+      line = textIn.ReadLine();
+      textOut << line + wxT("\n");
     }
-    wxMemoryOutputStream output;
-    wxTextOutputStream textOut(output);
-    if(output.IsOk())
-    {
-      wxMemoryInputStream mstream(
-        m_gnuplotSource_Compressed.GetData(),
-        m_gnuplotSource_Compressed.GetDataLen()
-        );
-      wxZlibInputStream zstream(mstream);
-      wxTextInputStream textIn(zstream);
-      wxString line;
-  
-      while(!zstream.Eof())
-      {
-        line = textIn.ReadLine();
-        textOut << line + wxT("\n");
-      }
-      textOut.Flush();
+    textOut.Flush();
 
-      retval.AppendData(output.GetOutputStreamBuffer()->GetBufferStart(),
-                        output.GetOutputStreamBuffer()->GetBufferSize());
-    }
+    retval.AppendData(output.GetOutputStreamBuffer()->GetBufferStart(),
+                      output.GetOutputStreamBuffer()->GetBufferSize());
   }
-  #ifdef HAVE_OMP_HEADER
-  omp_unset_lock(&m_gnuplotLock);
-  #endif
 
   return retval;
 }
@@ -511,28 +478,57 @@ wxMemoryBuffer Image::GetGnuplotSource()
 wxMemoryBuffer Image::GetGnuplotData()
 {
   wxMemoryBuffer retval;
-  #ifdef HAVE_OMP_HEADER
-  omp_set_lock(&m_gnuplotLock);
-  #else
-  #ifdef HAVE_OPENMP_TASKS
-  #pragma omp taskwait
-  #endif
-  #endif
+  if(
+    (m_gnuplotSource_Compressed.GetDataLen() < 2) || 
+    (m_gnuplotData_Compressed.GetDataLen() < 2))
   {
-    if(
-      (m_gnuplotSource_Compressed.GetDataLen() < 2) || 
-      (m_gnuplotData_Compressed.GetDataLen() < 2))
-    {
-      #ifdef HAVE_OMP_HEADER
-      omp_unset_lock(&m_gnuplotLock);
-      #endif
-      return retval;
-    }
+    return retval;
+  }
     
-    wxMemoryOutputStream output;
+  wxMemoryOutputStream output;
+  wxTextOutputStream textOut(output);
+  if(output.IsOk())
+  {
+    wxMemoryInputStream mstream(
+      m_gnuplotData_Compressed.GetData(),
+      m_gnuplotData_Compressed.GetDataLen()
+      );
+    wxZlibInputStream zstream(mstream);
+    wxTextInputStream textIn(zstream);
+    wxString line;
+  
+    while(!zstream.Eof())
+    {
+      line = textIn.ReadLine();
+      textOut << line + wxT("\n");
+    }
+    textOut.Flush();
+
+    retval.AppendData(output.GetOutputStreamBuffer()->GetBufferStart(),
+                      output.GetOutputStreamBuffer()->GetBufferSize());
+  }
+  return retval;
+}
+
+wxString Image::GnuplotData()
+{
+  if((!m_gnuplotData.IsEmpty()) && (!wxFileExists(m_gnuplotData)))
+  {
+    // Move the gnuplot data and data file into our temp directory
+    wxFileName gnuplotSourceFile(m_gnuplotSource);
+    m_gnuplotSource = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotSourceFile.GetFullName();
+    wxFileName gnuplotDataFile(m_gnuplotData);
+    m_gnuplotData = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotDataFile.GetFullName();
+
+    wxFileOutputStream output(m_gnuplotData);
     wxTextOutputStream textOut(output);
     if(output.IsOk())
     {
+      if(m_gnuplotData_Compressed.GetDataLen() <= 1)
+      {
+        wxLogMessage(_("No gnuplot data!"));
+        return wxEmptyString;
+      }
       wxMemoryInputStream mstream(
         m_gnuplotData_Compressed.GetData(),
         m_gnuplotData_Compressed.GetDataLen()
@@ -547,64 +543,8 @@ wxMemoryBuffer Image::GetGnuplotData()
         textOut << line + wxT("\n");
       }
       textOut.Flush();
-
-      retval.AppendData(output.GetOutputStreamBuffer()->GetBufferStart(),
-                        output.GetOutputStreamBuffer()->GetBufferSize());
     }
   }
-  #ifdef HAVE_OMP_HEADER
-  omp_unset_lock(&m_gnuplotLock);
-  #endif
-  return retval;
-}
-
-wxString Image::GnuplotData()
-{
-  if((!m_gnuplotData.IsEmpty()) && (!wxFileExists(m_gnuplotData)))
-  {
-    #ifdef HAVE_OMP_HEADER
-    omp_set_lock(&m_gnuplotLock);
-    #else
-    #ifdef HAVE_OPENMP_TASKS
-    #pragma omp taskwait
-    #endif
-    #endif
-    {
-    // Move the gnuplot data and data file into our temp directory
-      wxFileName gnuplotSourceFile(m_gnuplotSource);
-      m_gnuplotSource = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotSourceFile.GetFullName();
-      wxFileName gnuplotDataFile(m_gnuplotData);
-      m_gnuplotData = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotDataFile.GetFullName();
-
-      wxFileOutputStream output(m_gnuplotData);
-      wxTextOutputStream textOut(output);
-      if(output.IsOk())
-      {
-        if(m_gnuplotData_Compressed.GetDataLen() <= 1)
-        {
-          wxLogMessage(_("No gnuplot data!"));
-          return wxEmptyString;
-        }
-        wxMemoryInputStream mstream(
-          m_gnuplotData_Compressed.GetData(),
-          m_gnuplotData_Compressed.GetDataLen()
-          );
-        wxZlibInputStream zstream(mstream);
-        wxTextInputStream textIn(zstream);
-        wxString line;
-  
-        while(!zstream.Eof())
-        {
-          line = textIn.ReadLine();
-          textOut << line + wxT("\n");
-        }
-        textOut.Flush();
-      }
-    }
-  }
-  #ifdef HAVE_OMP_HEADER
-  omp_unset_lock(&m_gnuplotLock);
-  #endif
   return m_gnuplotData;
 }
 
@@ -612,54 +552,42 @@ wxString Image::GnuplotSource()
 {
   if((!m_gnuplotSource.IsEmpty()) && (!wxFileExists(m_gnuplotSource)))
   {
-    #ifdef HAVE_OMP_HEADER
-    omp_set_lock(&m_gnuplotLock);
-    #else
-    #ifdef HAVE_OPENMP_TASKS
-    #pragma omp taskwait
-    #endif
-    #endif
-    {
-      // Move the gnuplot source and data file into our temp directory
-      wxFileName gnuplotSourceFile(m_gnuplotSource);
-      m_gnuplotSource = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotSourceFile.GetFullName();
-      wxFileName gnuplotDataFile(m_gnuplotData);
-      m_gnuplotData = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotDataFile.GetFullName();
+    // Move the gnuplot source and data file into our temp directory
+    wxFileName gnuplotSourceFile(m_gnuplotSource);
+    m_gnuplotSource = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotSourceFile.GetFullName();
+    wxFileName gnuplotDataFile(m_gnuplotData);
+    m_gnuplotData = wxStandardPaths::Get().GetTempDir() + "/" + gnuplotDataFile.GetFullName();
   
-      wxFileOutputStream output(m_gnuplotSource);
-      wxTextOutputStream textOut(output);
-      if(output.IsOk())
-      {
+    wxFileOutputStream output(m_gnuplotSource);
+    wxTextOutputStream textOut(output);
+    if(output.IsOk())
+    {
 
-        if(m_gnuplotSource_Compressed.GetDataLen() <= 1)
-        {
-          wxLogMessage(_("No gnuplot source!"));
-          return wxEmptyString;
-        }
-        wxMemoryInputStream mstream(
-          m_gnuplotSource_Compressed.GetData(),
-          m_gnuplotSource_Compressed.GetDataLen()
-          );
-        wxZlibInputStream zstream(mstream);
-        if(zstream.IsOk())
-        {
-          wxTextInputStream textIn(zstream);
-          wxString line;
+      if(m_gnuplotSource_Compressed.GetDataLen() <= 1)
+      {
+        wxLogMessage(_("No gnuplot source!"));
+        return wxEmptyString;
+      }
+      wxMemoryInputStream mstream(
+        m_gnuplotSource_Compressed.GetData(),
+        m_gnuplotSource_Compressed.GetDataLen()
+        );
+      wxZlibInputStream zstream(mstream);
+      if(zstream.IsOk())
+      {
+        wxTextInputStream textIn(zstream);
+        wxString line;
           
-          while(!zstream.Eof())
-          {
-            line = textIn.ReadLine();
-            line.Replace(wxT("'<DATAFILENAME>'"),wxT("'")+m_gnuplotData+wxT("'"));
-            textOut << line + wxT("\n");
-          }
-          textOut.Flush();
+        while(!zstream.Eof())
+        {
+          line = textIn.ReadLine();
+          line.Replace(wxT("'<DATAFILENAME>'"),wxT("'")+m_gnuplotData+wxT("'"));
+          textOut << line + wxT("\n");
         }
+        textOut.Flush();
       }
     }
   }
-  #ifdef HAVE_OMP_HEADER
-  omp_unset_lock(&m_gnuplotLock);
-  #endif
   // Restore the data file, as well.
   GnuplotData();
   return m_gnuplotSource;
@@ -667,9 +595,6 @@ wxString Image::GnuplotSource()
  
 wxSize Image::ToImageFile(wxString filename)
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
   wxFileName fn(filename);
   wxString ext = fn.GetExt();
   if (filename.Lower().EndsWith(GetExtension().Lower()))
@@ -745,9 +670,6 @@ wxBitmap Image::GetBitmap(double scale)
 {
   // Recalculate contains its own WaitForLoad object.
   Recalculate(scale);
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
 
   if(!m_isOk)
   {
@@ -768,9 +690,6 @@ wxBitmap Image::GetBitmap(double scale)
     nsvgRasterize(m_svgRast.get(), m_svgImage, 0,0,
                   ((double)m_width)/((double)m_originalWidth),
                   imgdata.data(), m_width, m_height, m_width*4);
-    #ifdef HAVE_OMP_HEADER
-    omp_unset_lock(&m_gnuplotLock);
-    #endif
     return m_scaledBitmap = SvgBitmap::RGBA2wxBitmap(imgdata.data(), m_width, m_height);
   }
   else
@@ -803,16 +722,13 @@ wxBitmap Image::GetBitmap(double scale)
   }
   else
     m_scaledBitmap = wxBitmap(1,1);
-  #ifdef HAVE_OMP_HEADER
-  omp_unset_lock(&m_gnuplotLock);
-  #endif
   return m_scaledBitmap;
 }
 
 void Image::InvalidBitmap()
 {
   m_isOk = false;
-  m_width = 800; m_height = 600;
+  m_width = 800 * m_ppi / 96; m_height = 600 * m_ppi / 96;
   // Create a "image not loaded" bitmap.
   m_scaledBitmap.Create(m_width, m_height);
   
@@ -834,14 +750,10 @@ void Image::InvalidBitmap()
   
   dc.GetTextExtent(error, &width, &height);
   dc.DrawText(error, (m_width - width) / 2, (m_height - height) / 2);
-  m_scaledBitmap = m_scaledBitmap.ConvertToImage();
 }
 
 void Image::LoadImage(const wxBitmap &bitmap)
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
   // Convert the bitmap to a png image we can use as m_compressedImage
   wxImage image = bitmap.ConvertToImage();
   m_isOk = image.IsOk();
@@ -872,25 +784,12 @@ void Image::LoadImage(wxString image, std::shared_ptr<wxFileSystem> filesystem,
   m_fs_keepalive_imagedata = filesystem;
   m_extension = wxFileName(image).GetExt();
   m_extension = m_extension.Lower();
-  // If we don't have fine-grained locking using omp.h we don't profit from sending the
-  // load process to the background and therefore load images from the main thread.
-  // Loading images is of rather high priority as they are needed during the
-  // recalculation that follows
-  #ifdef HAVE_OMP_HEADER
-  wxLogMessage(_("Scheduling background task that loads an image"));
-  #if HAVE_OPENMP_TASKS
-  #pragma omp task
-  #endif
-  #endif
   m_imageName = image;
   LoadImage_Backgroundtask(image, filesystem, remove);
 }
 
 void Image::LoadImage_Backgroundtask(wxString image, std::shared_ptr<wxFileSystem> filesystem, bool remove)
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
 
   m_compressedImage.Clear();
   m_scaledBitmap.Create(1, 1);
@@ -898,9 +797,6 @@ void Image::LoadImage_Backgroundtask(wxString image, std::shared_ptr<wxFileSyste
   if (filesystem)
   {
     wxFSFile * fsfile;
-    #ifdef HAVE_OPENMP_TASKS
-    #pragma omp critical (OpenFSFile)
-    #endif
     fsfile = filesystem->OpenFile(image);
     if (fsfile)
     { // open successful
@@ -982,7 +878,7 @@ void Image::LoadImage_Backgroundtask(wxString image, std::shared_ptr<wxFileSyste
           svgContents_string += line + wxT("\n");
         }
       }
-      // Convert the data we have read to a modifyable char * containing the svg file's contents.
+      // Convert the data we have read to a modifiable char * containing the svg file's contents.
       char *svgContents;
       svgContents = (char *)strdup(svgContents_string.utf8_str());
 
@@ -1021,6 +917,18 @@ void Image::LoadImage_Backgroundtask(wxString image, std::shared_ptr<wxFileSyste
         m_originalWidth = Image.GetWidth();
         m_originalHeight = Image.GetHeight();
         m_isOk = true;
+        if(Image.HasOption(wxT("wxIMAGE_OPTION_RESOLUTION")))
+        {
+          int resolution;
+          resolution = Image.GetOptionInt(wxT("wxIMAGE_OPTION_RESOLUTION"));
+          if(Image.HasOption(wxT("wxIMAGE_OPTION_RESOLUTIONUNIT")))
+          {
+            if(Image.GetOptionInt("wxIMAGE_OPTION_RESOLUTIONUNIT") == wxIMAGE_RESOLUTION_CM)
+              resolution *= 2.54;
+          }
+          if(resolution > 50)
+            m_ppi = resolution;
+        }
       }
       else
       {
@@ -1029,20 +937,14 @@ void Image::LoadImage_Backgroundtask(wxString image, std::shared_ptr<wxFileSyste
     }
   }
   m_fs_keepalive_imagedata.reset();
-  #ifdef HAVE_OMP_HEADER
-  omp_unset_lock(&m_imageLoadLock);
-  #endif
 }
 
 void Image::Recalculate(double scale)
 {
-  #ifdef HAVE_OMP_HEADER
-  WaitForLoad waitforload(&m_imageLoadLock);
-  #endif
   int width = m_originalWidth;
   int height = m_originalHeight;
   Configuration *configuration = (*m_configuration);
-
+  
   // We want the image to get bigger if the user zooms in - and
   // if a high printing resolution requires us to scale everything up.
   // To also take care of the user's printing-scale,
@@ -1051,6 +953,11 @@ void Image::Recalculate(double scale)
   // Ensure a minimum size for images.
   if (scale < 0.01) scale = 0.01;
 
+
+  // pre-scale the image according to the current output's ppi and the image's ppi;
+  scale *= (*m_configuration)->GetPPI().x;
+  scale /= m_ppi;
+
   if ((width < 1) || (height < 1))
   {
     m_width = 700;
@@ -1065,13 +972,6 @@ void Image::Recalculate(double scale)
     viewPortHeight = 10;
   if (viewPortWidth < 10)
     viewPortWidth = 10;
-
-  if (!configuration->GetPrinting())
-  {
-    // Change the scale only if we are not printing.
-    // Resetting the scale for printing makes the images become too small.
-    scale = 1.0;
-  }
   
   // Shrink to .9* the canvas size, if needed
   if (scale * width > .9 * viewPortWidth)
@@ -1084,10 +984,10 @@ void Image::Recalculate(double scale)
   }
 
   // Shrink to be smaller than the maximum size.
-  if ((m_maxWidth > 0) && (scale * width > m_maxWidth * (*m_configuration)->GetDC()->GetPPI().x))
-    scale = m_maxWidth * (*m_configuration)->GetDC()->GetPPI().x / width;
-  if ((m_maxHeight > 0) && (scale * height > m_maxHeight * (*m_configuration)->GetDC()->GetPPI().y))
-    scale = m_maxHeight * (*m_configuration)->GetDC()->GetPPI().y / height;
+  if ((m_maxWidth > 0) && (scale * width > m_maxWidth * (*m_configuration)->GetPPI().x))
+    scale = m_maxWidth * (*m_configuration)->GetPPI().x / width;
+  if ((m_maxHeight > 0) && (scale * height > m_maxHeight * (*m_configuration)->GetPPI().y))
+    scale = m_maxHeight * (*m_configuration)->GetPPI().y / height;
   
   // Set the width of the scaled image
   m_height = (int) (scale * height);
@@ -1095,13 +995,12 @@ void Image::Recalculate(double scale)
 
   if((m_height < 1) || (m_width < 1))
   {
-    m_height = 100;
-    m_width = 100;
+    m_height = 1;
+    m_width = 1;
   }
   // Clear this cell's image cache if it doesn't contain an image of the size
-  // we need right now. Printing uses unscaled bitmaps, so the cache is left
-  // unchanged then.
-  if (!configuration->GetPrinting() && m_scaledBitmap.GetWidth() != m_width)
+  // we need right now.
+  if (m_scaledBitmap.GetWidth() != m_width)
     ClearCache();
 }
 
