@@ -27,13 +27,13 @@
 #include "StatusBar.h"
 #include <wx/artprov.h>
 #include <wx/display.h>
-#include "../art/statusbar/images.h"
 #include "SvgBitmap.h"
 #include <wx/mstream.h>
 #include <wx/wfstream.h>
 #include <wx/zstream.h>
 #include <wx/txtstrm.h>
 #include "Image.h"
+#include "wxm_statusbar_images.h"
 
 StatusBar::StatusBar(wxWindow *parent, int id) : wxStatusBar(parent, id),
                                                  m_ppi(wxSize(-1,-1))
@@ -72,7 +72,7 @@ void StatusBar::UpdateBitmaps()
   wxSize ppi(-1,-1);
 #if wxCHECK_VERSION(3, 1, 1)
   wxDisplay display;
-  
+
   int display_idx = wxDisplay::GetFromWindow(GetParent());
   if (display_idx < 0)
     ppi = wxSize(72,72);
@@ -92,24 +92,18 @@ void StatusBar::UpdateBitmaps()
   {
     m_ppi = ppi;
     m_network_error = GetImage("network-error",
-                               network_error_svg_gz,network_error_svg_gz_len
-      );
+                               NETWORK_ERROR_SVG_GZ, NETWORK_ERROR_SVG_GZ_SIZE);
     m_network_offline = GetImage("network-offline",
-                                 network_offline_svg_gz,network_offline_svg_gz_len
-      );
+                                 NETWORK_OFFLINE_SVG_GZ, NETWORK_OFFLINE_SVG_GZ_SIZE);
     m_network_transmit = GetImage("network-transmit",
-                                  network_transmit_svg_gz,network_transmit_svg_gz_len
-      );
+                                  NETWORK_TRANSMIT_SVG_GZ, NETWORK_TRANSMIT_SVG_GZ_SIZE);
     m_network_idle = GetImage("network-idle",
-                              network_idle_svg_gz,network_idle_svg_gz_len
-      );
+                              NETWORK_IDLE_SVG_GZ, NETWORK_IDLE_SVG_GZ_SIZE);
     m_network_idle_inactive = wxBitmap(m_network_idle.ConvertToImage().ConvertToDisabled());
     m_network_receive = GetImage("network-receive",
-                                 network_receive_svg_gz,network_receive_svg_gz_len
-      );
+                                 NETWORK_RECEIVE_SVG_GZ, NETWORK_RECEIVE_SVG_GZ_SIZE);
     m_network_transmit_receive = GetImage("network-transmit-receive",
-                                          network_transmit_receive_svg_gz,network_transmit_receive_svg_gz_len
-      );
+                                          NETWORK_TRANSMIT_RECEIVE_SVG_GZ, NETWORK_TRANSMIT_RECEIVE_SVG_GZ_SIZE);
   }
 }
 
@@ -177,10 +171,16 @@ void StatusBar::NetworkStatus(networkState status)
 //      if(status != m_oldNetworkState)
 //        m_maximaPercentage = m_oldmaximaPercentage = -1;
       if(m_maximaPercentage != 0)
-        m_networkStatus->SetBitmap(m_network_idle);
+      {
+        if((!!m_maximaPercentage) != (!!m_oldmaximaPercentage))
+          m_networkStatus->SetBitmap(m_network_idle);
+      }
       else
-        m_networkStatus->SetBitmap(m_network_idle_inactive);
-      
+      {
+        if((!!m_maximaPercentage) != (!!m_oldmaximaPercentage))
+          m_networkStatus->SetBitmap(m_network_idle_inactive);
+      }
+
       m_networkState = status;
       wxString toolTip = m_stdToolTip;
       if(m_maximaPercentage >= 0)
@@ -192,21 +192,26 @@ void StatusBar::NetworkStatus(networkState status)
     }
     break;
     case error:
-      m_networkStatus->SetBitmap(m_network_error);
-      m_networkState = status;
-      m_networkStatus->SetToolTip(m_networkErrToolTip);
+      if((status != m_oldNetworkState))
+      {
+        m_networkStatus->SetBitmap(m_network_error);
+        m_networkState = status;
+        m_networkStatus->SetToolTip(m_networkErrToolTip);
+      }
       break;
     case offline:
-      m_networkStatus->SetBitmap(m_network_offline);
-      m_networkState = status;
+      if((status != m_oldNetworkState))
+      {
+        m_networkStatus->SetBitmap(m_network_offline);
+        m_networkState = status;
+      }
       m_networkStatus->SetToolTip(m_noConnectionToolTip);
       break;
     case receive:
     {
       ReceiveTimer.StartOnce(200);
       HandleTimerEvent();
-      if((m_oldmaximaPercentage >= 0) &&(m_maximaPercentage < 0))
-        m_networkStatus->SetToolTip(m_stdToolTip);
+      m_networkStatus->SetToolTip(m_stdToolTip);
     }
     break;
     case transmit:
@@ -236,15 +241,13 @@ void StatusBar::OnSize(wxSizeEvent &event)
   event.Skip();
 }
 
-#define ABS(val) ((val) >= 0 ? (val) : -(val))
-
 wxBitmap StatusBar::GetImage(wxString name,
                           unsigned char *data, size_t len)
 {
   wxSize ppi;
 #if wxCHECK_VERSION(3, 1, 1)
   wxDisplay display;
-  
+
   int display_idx = wxDisplay::GetFromWindow(GetParent());
   if (display_idx < 0)
     ppi = wxSize(72,72);
@@ -265,7 +268,7 @@ wxBitmap StatusBar::GetImage(wxString name,
     targetWidth = 16;
   if(targetHeight < 16)
     targetHeight = 16;
-  
+
   wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_TOOLBAR, wxSize(targetWidth, targetHeight));
   wxImage img;
 
@@ -284,7 +287,7 @@ wxBitmap StatusBar::GetImage(wxString name,
   }
 
   if(!img.IsOk())
-    return SvgBitmap(data, len, targetWidth, targetWidth);
+    return SvgBitmap(this, data, len, targetWidth, targetWidth);
 
   targetWidth = static_cast<double>(GetSize().GetHeight());
   targetHeight = static_cast<double>(GetSize().GetHeight());
