@@ -32,6 +32,7 @@
 
 #include "precomp.h"
 #include <wx/bitmap.h>
+#define NANOSVG_ALL_COLOR_KEYWORDS
 #include "nanoSVG/nanosvg.h"
 #include "nanoSVG/nanosvgrast.h"
 
@@ -41,8 +42,8 @@ class SvgBitmap: public wxBitmap
 {
 public:
 //! A constructor that loads the data into a wxBitmap
-  SvgBitmap(const unsigned char *data, size_t len, int width = 640, int height = 480, int scaleFactor = 1);
-  SvgBitmap(const unsigned char *data, size_t len, wxSize siz, int scaleFactor = 1);
+  SvgBitmap(wxWindow *window, const unsigned char *data, size_t len, int width = 640, int height = 480, int scaleFactor = 1);
+  SvgBitmap(wxWindow *window, const unsigned char *data, size_t len, wxSize siz, int scaleFactor = 1);
   ~SvgBitmap() override;
   SvgBitmap(SvgBitmap &&) = delete;
   SvgBitmap &operator=(SvgBitmap &&o);
@@ -56,15 +57,8 @@ public:
   //! Gets the original size of the svg image
   wxSize GetOriginalSize() const
   { return m_svgImage ? wxSize(m_svgImage->width, m_svgImage->height) : wxDefaultSize; }
-  /*! An "invalid bitmap" sign
-    
-    We should make the image static and generate it on start-up 
-    so whenever we need it it already would be available as a 
-    refcounted object and therefore be likely to 
-    be available even if the problem was a failing malloc(). 
-    But with wx3.0 gtk crashes in this case.
-   */
-  static wxBitmap GetInvalidBitmap(int targetSize);
+  /*! An "invalid bitmap" sign */
+  wxBitmap GetInvalidBitmap(int targetSize);
 private:
   //! The scale factor the bitmap is created with
   int m_scaleFactor;
@@ -72,6 +66,8 @@ private:
   static struct NSVGrasterizer* m_svgRast;
   //! The renderable svg image after we have read it in
   std::unique_ptr<NSVGimage, decltype(std::free)*> m_svgImage{nullptr, std::free};
+  //! The window this bitmap will be drawn on
+  wxWindow *m_window;
 };
 
 #endif // SVGBITMAP_H
