@@ -22,10 +22,10 @@
 //  SPDX-License-Identifier: GPL-2.0+
 
 /*!
- * \file This file implements the text style system.
+ * \file
+ * This file implements the text style system.
  */
 
-#define wxNO_UNSAFE_WXSTRING_CONV
 #include "TextStyle.h"
 #include "FontCache.h"
 #include <wx/fontenum.h>
