@@ -27,6 +27,7 @@
 //  SPDX-License-Identifier: GPL-2.0+
 
 #include "TipOfTheDay.h"
+#include "wxm_tipoftheday_images.h"
 #include <wx/config.h>
 #include <wx/display.h>
 #include <wx/persist.h>
@@ -37,9 +38,9 @@
 #include <wx/zstream.h>
 #include <wx/txtstrm.h>
 #include "Image.h"
+#include <cstdlib>
 
 #define ICON_SCALE (0.35)
-#define ABS(val) ((val) >= 0 ? (val) : -(val))
 
 TipOfTheDay::TipOfTheDay(wxWindow *parent)
   : wxDialog(parent,-1,_("Tip of the day"))
@@ -51,7 +52,10 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("By default, Shift-Enter is used to evaluate commands, while Enter is used for multiline input. This behaviour can be changed in 'Edit->Configure' dialog by checking 'Enter evaluates cells'. This switches the roles of these two key commands.")
     );
   m_tips.Add(
-    _("Maxima uses ':' to set values ('a : 3;') and ':=' to define functions ('f(x) := x^2;').")
+    _("Maxima uses ':' to assign values to a variable ('a : 3;'), not '=' as most programming languages do.\nThis allows, that a mathematical equation can be assigned to a variable, e.g.\n eq1:3*x=4;\nwhich may then be solved for x with\nsolve(eq1,x);")
+    );
+  m_tips.Add(
+    _("Maxima uses ':=' to define functions, e.g.:\nf(x) := x^2;")
     );
   m_tips.Add(
     _("You can access the last output using the variable '%'. You can access the output of previous commands using variables '%on' where n is the number of output.")
@@ -66,7 +70,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("You can insert different types of 'cells' in wxMaxima documents using the 'Cell' menu. Note that only 'input cells' can be evaluated, while others are used for commenting and structuring your calculations.")
     );
   m_tips.Add(
-    _("A new document format has been introduced in wxMaxima 0.8.2 that saves not only your input and text commentaries, but also the outputs of your calculations. When saving your document, select 'wxMaxima XML document' format.")
+    _("A new document format has been introduced in wxMaxima 0.8.2 that saves not only your input and text commentaries, but also the outputs of your calculations. When saving your document, select 'Whole document (*.wxmx)' format.")
     );
   m_tips.Add(
     _("Title, section and subsection cells can be folded to hide their contents. To fold or unfold, click in the square next to the cell. If you shift-click, all sublevels of that cell will also fold/unfold.")
@@ -78,7 +82,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("There are many resources about Maxima and wxMaxima on the internet. Visit https://wxMaxima-developers.github.io/wxmaxima/help.html for more information and to find tutorials on using wxMaxima and Maxima.")
     );
   m_tips.Add(
-    _("You can get help on a Maxima function by selecting or clicking on the function name and pressing F1. wxMaxima will search help index for the selection or the word under the cursor.")
+    _("You can get help on a Maxima function by selecting or clicking on the function name and pressing F1. wxMaxima will search the help index for the selection or the word under the cursor.")
     );
   m_tips.Add(
     _("A 'horizontal cursor' was introduced in wxMaxima 0.8.0. It looks like a horizontal line between cells. It indicates where a new cell will appear if you type or paste text or execute a menu command.")
@@ -114,7 +118,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("To save the size and position of wxMaxima windows between session, use 'Edit->Configure' dialog.")
     );
   m_tips.Add(
-    _("Since wxMaxima 0.8.2 you can also insert images into your documents. Use 'Cell->Insert Image...' menu command. Note that you have to save your document in 'wxMaxima XML document' format if you want the image to be saved along with your document.")
+    _("Since wxMaxima 0.8.2 you can also insert images into your documents. Use 'Cell->Insert Image...' menu command.")
     );
   m_tips.Add(
     _("Besides the global undo functionality that is active when the cursor is between cells, wxMaxima has a per-cell undo function that is active if the cursor is inside a cell. Pressing Ctrl+Z inside a cell can therefore been used for a fine-pitch undo that doesn't affect latter changes made in other cells.")
@@ -123,16 +127,16 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("Pressing Ctrl+Space or Ctrl+Tab starts an autocomplete function that can not only complete all functions that are integrated into the Maxima core and their parameters: It also knows about parameters from currently loaded packages and from functions that are defined in the current file.")
     );
   m_tips.Add(
-    _("It is possible to define reusable Maxima libraries with wxMaxima that can be later loaded by using the load() function. All that has to be done is to export a file in the .mac or to save it in the .wxm format. Note, though, that a few special characters like the \"not equal\" symbol for \"#\" are handled by wxMaxima, not by maxima and therefore cannot be recognized on load().")
+    _("It is possible to define reusable Maxima libraries with wxMaxima that can be later loaded by using the load() function. All that has to be done is to export a file in the .mac or to save it in the .wxm format. Note, though, that a few special characters like the \"not equal\" symbol for \"#\" are handled by wxMaxima, not by Maxima and therefore cannot be recognized on load().")
     );
   m_tips.Add(
-    _("wxMaxima can be made to execute commands at every start-up by placing them in a text file with the name wxmaxima.rc in the user directory. This directory can be found by typing maxima_userdir")
+    _("wxMaxima can be made to execute commands at every start-up by placing them in a text file with the name wxmaxima.rc in the user directory. This directory can be found by typing maxima_userdir;")
     );
   m_tips.Add(
-    _("Libraries can be accessed by any wxMaxima process regardless of which directory it runs in, if they are placed in the user directory. This directory can be found by typing maxima_userdir")
+    _("Libraries can be accessed by any wxMaxima process regardless of which directory it runs in, if they are placed in the user directory. This directory can be found by typing maxima_userdir;")
     );
   m_tips.Add(
-    _("The load() command of Maxima versions >5.38 is able to load .wxm files as library of maxima's commands.")
+    _("The load() command of Maxima versions >5.38 is able to load .wxm files as library of Maxima's commands.")
     );
   m_tips.Add(
     _("A plot to be embedded into the work sheet by preceding its name with a \"wx\". \"draw\" can be replaced by \"wxdraw\", plot by \"wxplot\" etc.")
@@ -141,7 +145,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("wxMaxima provides the \"table_form\" command that shows lists in an alternative form. One example would be:\n\ntable_form([1,2,3,4,5]);")
     );
   m_tips.Add(
-    _("In order to visualize how a parameter affects an equation wxmaxima provides commands that start with \"with_slider_\" and that create animations. One example would be:\n\n    with_slider_draw(\n        a,[-16,-9,-4,-2,0,1,2,3,4,5,6,7],\n        title=concat(\"a=\",a),\n        grid=true,\n        explicit(\n            x^2-a*x,\n            x,-10,10\n        )\n    );")
+    _("In order to visualize how a parameter affects an equation wxMaxima provides commands that start with \"with_slider_\" and that create animations. One example would be:\n\n    with_slider_draw(\n        a,[-16,-9,-4,-2,0,1,2,3,4,5,6,7],\n        title=concat(\"a=\",a),\n        grid=true,\n        explicit(\n            x^2-a*x,\n            x,-10,10\n        )\n    );")
     );
   m_tips.Add(
     _("If the \"Autosave\" functionality is enabled in the configuration dialogue wxMaxima acts like mobile apps that backup the file every few minutes and save it on closing.")
@@ -165,10 +169,10 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("The rhs() (\"right hand side\") command allows to retrieve the result of an equation in exactly the format a function would have:\n\n    Values:[\n        /* m=1.2 tons */\n        m=1.2*10^3,\n        /* 100 km/h*/\n        v=100*10^3/(60*60)\n    ];\n    Energy:W=1/2*m*v^2;\n    at(Energy,Values);\n    W_mech:rhs(%);")
     );
   m_tips.Add(
-    _("In text cells Bullet lists can be created by beginning a line with \" * \". The number of spaces in front of the \"*\" determines the indentation level; Indentation can be continued in the next line by indenting the line using spaces.")
+    _("In text cells bullet lists can be created by beginning a line with \" * \". The number of spaces in front of the \"*\" determines the indentation level; Indentation can be continued in the next line by indenting the line using spaces.")
     );
   m_tips.Add(
-    _("Text cells can hold Citations that as marked as such by beginning a line with \" > \". The number of spaces in front of the \">\" determines the indentation level, as it does with bullet lists.")
+    _("Text cells can hold citations that as marked as such by beginning a line with \" > \". The number of spaces in front of the \">\" determines the indentation level, as it does with bullet lists.")
     );
   m_tips.Add(
     _("The key combination Shift+Space results in a non-breakable space.")
@@ -179,13 +183,33 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
   m_tips.Add(
     _("wxMaxima needs more developers. It is an open source project where you can contribute. Join the wxMaxima development at: https://github.com/wxMaxima-developers/wxmaxima")
     );
+  m_tips.Add(
+    _("You found a bug in wxMaxima? Please report it at https://github.com/wxMaxima-developers/wxmaxima/issues\nA mathematical bug is maybe a bug in Maxima (the backend), which can be reported at https://sourceforge.net/p/maxima/bugs/\nPlease check, before, if the bug was not already reported.")
+    );
+  m_tips.Add(
+    _("You got an idea for an improvement of wxMaxima? Great! Please submit your idea at https://github.com/wxMaxima-developers/wxmaxima/issues. ")
+    );
+  m_tips.Add(
+    _("wxMaxima is programmed in C++ using the wxWidgets toolkit and uses CMake as build system. You can contribute to wxMaxima, if you have some knowledge with this environment.")
+    );
+  m_tips.Add(
+    _("While wxMaxima is programmed mostly in C++, it also uses some Lisp code. You can help to improve wxMaxima, if you have knowledge of Common Lisp.")
+    );
+  m_tips.Add(
+    _("If you have ideas, how to improve the design or functionality of wxMaxima, please share your ideas at the issue list or discussion forum at https://github.com/wxMaxima-developers/wxmaxima/.")
+    );
+  m_tips.Add(
+    _("There is a discussion forum for wxMaxima at https://github.com/wxMaxima-developers/wxmaxima/discussions.\nYou can ask questions there - and please help other users, by answering their questions, if you know an answer.")
+    );
 
   m_num = 0;
   wxConfigBase *config = wxConfig::Get();
-  config->Read(wxT("tipNum"), &m_num);
-  if(m_num < 0)
-    m_num = m_tips.GetCount()-1;
-  if((unsigned)m_num >=m_tips.GetCount())
+
+  // If nm is negative the if will make it posigive.
+  long nm;
+  config->Read(wxT("tipNum"), &nm);
+  m_num = nm;
+  if(m_num >=m_tips.GetCount())
     m_num = 0;
 
   SetName("TipOfTheDay");
@@ -194,9 +218,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
   wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
   wxButton *backButton = new wxButton(this,-1);
   backButton->SetBitmap(
-    GetImage(
-      media_playback_start_reverse_svg_gz,media_playback_start_reverse_svg_gz_len
-      )
+    GetImage(MEDIA_PLAYBACK_START_REVERSE1_SVG_GZ, MEDIA_PLAYBACK_START_REVERSE1_SVG_GZ_SIZE)
     );
   backButton->Connect(
     wxEVT_BUTTON,
@@ -210,9 +232,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
             wxSizerFlags().Proportion(10).Center());
   wxButton *forwardButton = new wxButton(this,-1);
   forwardButton->SetBitmap(
-    GetImage(
-      media_playback_start_svg_gz,media_playback_start_svg_gz_len
-      )
+    GetImage(MEDIA_PLAYBACK_START1_SVG_GZ, MEDIA_PLAYBACK_START1_SVG_GZ_SIZE)
     );
   forwardButton->Connect(
     wxEVT_BUTTON,
@@ -292,13 +312,13 @@ wxImage TipOfTheDay::GetImage(unsigned char *data, size_t len)
     sizeB >>= 1;
   }
 
-  if(ABS(targetSize - sizeA) < ABS(targetSize - sizeB)) {
+  if(std::abs(targetSize - sizeA) < std::abs(targetSize - sizeB)) {
     targetSize = sizeA;
   } else {
     targetSize = sizeB;
   }
 
-  wxImage img = SvgBitmap(data, len, targetSize, targetSize).ConvertToImage();
+  wxImage img = SvgBitmap(this, data, len, targetSize, targetSize).ConvertToImage();
 
 #if defined __WXMSW__
 #if wxCHECK_VERSION(3, 1, 1)
@@ -317,15 +337,16 @@ wxImage TipOfTheDay::GetImage(unsigned char *data, size_t len)
 void TipOfTheDay::OnNextButton(wxCommandEvent &WXUNUSED(dummy))
 {
   m_num++;
-  if((unsigned)m_num >=m_tips.GetCount())
+  if(m_num >=m_tips.GetCount())
     m_num = 0;
   m_tip->SetValue(m_tips[m_num]);
 }
 
 void TipOfTheDay::OnPreviousButton(wxCommandEvent &WXUNUSED(dummy))
 {
-  m_num--;
-  if(m_num < 0)
+  if(m_num > 0)
+    m_num--;
+  else
     m_num = m_tips.GetCount()-1;
   m_tip->SetValue(m_tips[m_num]);
 }
