@@ -57,7 +57,7 @@ protected:
 
 private:
   wxString GetTip(unsigned int n);
-  int m_num;
+  unsigned int m_num;
   wxTextCtrl *m_tip;
   wxString GetTip();
   wxCheckBox *m_showAtStartup;
