@@ -27,7 +27,7 @@
  */
 
 #include "ToolBar.h"
-#include "../art/toolbar/images.h"
+#include "wxm_toolbar_images.h"
 #include <wx/mstream.h>
 #include <wx/wfstream.h>
 #include <wx/display.h>
@@ -37,6 +37,7 @@
 #include <wx/zstream.h>
 #include <wx/txtstrm.h>
 #include "Image.h"
+#include <cstdlib>
 
 #if wxCHECK_VERSION(3, 1, 0)
 #define TOOLBAR_ICON_SCALE (0.25)
@@ -44,14 +45,12 @@
 #define TOOLBAR_ICON_SCALE (0.35)
 #endif
 
-#define ABS(val) ((val) >= 0 ? (val) : -(val))
-
 wxBitmap ToolBar::GetBitmap(wxString name, unsigned char *data, size_t len, wxSize siz)
 {
   if(siz.x <= 0)
   {
 #if wxCHECK_VERSION(3, 1, 1)
-    wxDisplay display;    
+    wxDisplay display;
     int display_idx = wxDisplay::GetFromWindow(GetParent());
     if (display_idx < 0)
       m_ppi = wxSize(72,72);
@@ -62,26 +61,26 @@ wxBitmap ToolBar::GetBitmap(wxString name, unsigned char *data, size_t len, wxSi
 #endif
     if((m_ppi.x <= 10) || (m_ppi.y <= 10))
       m_ppi = wxGetDisplayPPI();
-    
+
     if((m_ppi.x <= 10) || (m_ppi.y <= 10))
       m_ppi = wxSize(72,72);
-    
+
   #if defined __WXOSX__
     int targetSize = wxMax(m_ppi.x,75) * TOOLBAR_ICON_SCALE * GetContentScaleFactor();
-#else
+  #else
     int targetSize = wxMax(m_ppi.x,75) * TOOLBAR_ICON_SCALE;
-#endif
+  #endif
     int sizeA = 128 << 4;
     while(sizeA * 3 / 2 > targetSize && sizeA >= 32) {
       sizeA >>= 1;
     };
-    
+
     int sizeB = 192 << 4;
     while(sizeB * 4 / 3 > targetSize && sizeB >= 32) {
       sizeB >>= 1;
     }
-    
-    if(ABS(targetSize - sizeA) < ABS(targetSize - sizeB))
+
+    if(std::abs(targetSize - sizeA) < std::abs(targetSize - sizeB))
       targetSize = sizeA;
     else
       targetSize = sizeB;
@@ -92,8 +91,8 @@ wxBitmap ToolBar::GetBitmap(wxString name, unsigned char *data, size_t len, wxSi
   if(bmp.IsOk())
     img = bmp.ConvertToImage();
   if(!img.IsOk())
-    return SvgBitmap(data, len, siz, GetContentScaleFactor());
-  
+    return SvgBitmap(this, data, len, siz, GetContentScaleFactor());
+
   img.Rescale(siz.x, siz.y, wxIMAGE_QUALITY_HIGH);
 #if defined __WXOSX__
   return wxBitmap(img,wxBITMAP_SCREEN_DEPTH,GetContentScaleFactor());
@@ -107,17 +106,17 @@ ToolBar::~ToolBar()
   m_plotSlider = NULL;
 }
 
-void ToolBar::UpdateSlider(SlideShow *cell)
+void ToolBar::UpdateSlider(AnimationCell *cell)
 {
   if(cell == NULL)
     return;
-  int slideShowDisplayedIndex = cell->GetDisplayedIndex();
-  int slideShowMaxIndex = cell->Length();
+  int animationDisplayedIndex = cell->GetDisplayedIndex();
+  int animationMaxIndex = cell->Length();
 
-  if ((m_slideShowDisplayedIndex != slideShowDisplayedIndex) || (m_slideShowMaxIndex != slideShowMaxIndex))
+  if ((m_animationDisplayedIndex != animationDisplayedIndex) || (m_animationMaxIndex != animationMaxIndex))
   {
-    m_slideShowMaxIndex = slideShowMaxIndex;
-    m_slideShowDisplayedIndex = slideShowDisplayedIndex;
+    m_animationMaxIndex = animationMaxIndex;
+    m_animationDisplayedIndex = animationDisplayedIndex;
     if (m_plotSlider != NULL)
     {
       m_plotSlider->SetRange(0, cell->Length() - 1);
@@ -154,13 +153,13 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
   // do the following:
   Realize();
   SetInitialSize(wxSize(100000,GetBestSize().y));
-  Realize();  
+  Realize();
 }
 
 void ToolBar::AddTools()
 {
   Clear();
-  m_ppi = wxSize(-1,-1);  
+  m_ppi = wxSize(-1,-1);
   if(ShowNew())
     AddTool(wxID_NEW, _("New"),GetNewBitmap(),_("New document"));
   if(ShowOpenSave())
@@ -176,11 +175,27 @@ void ToolBar::AddTools()
 #endif
     AddTool(wxID_PRINT, _("Print"), GetPrintBitmap(), _("Print document"));
   }
+  if(ShowUndoRedo())
+  {
+#ifndef __WXOSX__
+    if(ShowOpenSave() || ShowNew())
+      AddSeparator();
+#endif
+    AddTool(wxID_UNDO, _("Undo"), GetUndoBitmap());
+    AddTool(wxID_REDO, _("Redo"), GetRedoBitmap());
+  }
   if(ShowOptions())
+  {
+#ifndef __WXOSX__
+    if(ShowOpenSave() || ShowNew() || ShowUndoRedo())
+      AddSeparator();
+#endif
     AddTool(wxID_PREFERENCES, _("Options"), GetPreferencesBitmap(), _("Configure wxMaxima"));
+  }
   if(ShowCopyPaste())
   {
 #ifndef __WXOSX__
+    if(ShowSelectAll() || ShowOpenSave() || ShowNew() || ShowPrint() || ShowUndoRedo())
     AddSeparator();
 #endif
     AddTool(wxID_CUT, _("Cut"), GetCutBitmap(), _("Cut selection"));
@@ -192,23 +207,23 @@ void ToolBar::AddTools()
   if(ShowSearch())
   {
 #ifndef __WXOSX__
-    if(ShowSelectAll() || ShowOpenSave() || ShowNew() || ShowPrint())
+    if(ShowSelectAll() || ShowOpenSave() || ShowNew() || ShowPrint() || ShowUndoRedo() || ShowCopyPaste())
       AddSeparator();
 #endif
     AddTool(wxID_FIND, _("Find"), GetFindBitmap(), _("Find and replace"));
   }
 #ifndef __WXOSX__
-  if(ShowSelectAll() || ShowOpenSave() || ShowNew() || ShowPrint() || ShowOptions())
+  if(ShowSelectAll() || ShowOpenSave() || ShowNew() || ShowPrint() || ShowOptions() || ShowUndoRedo() || ShowSearch())
     AddSeparator();
 #endif
   AddTool(menu_restart_id, _("Restart Maxima"), GetRestartBitmap(),
           _("Completely stop maxima and restart it"));
   AddTool(tb_interrupt, _("Interrupt"), GetInterruptBitmap(),
           _("Interrupt current computation. To completely restart maxima press the button left to this one."));
-  m_followIcon = GetBitmap(wxT("weather-clear"),
-                          weather_clear_svg_gz,weather_clear_svg_gz_len);
+  m_followIcon = GetBitmap(wxT("arrow_up_square"),
+                          ARROW_UP_SQUARE_SVG_GZ, ARROW_UP_SQUARE_SVG_GZ_SIZE);
   m_needsInformationIcon = GetBitmap(wxT("software-update-urgent"),
-                                    software_update_urgent_svg_gz,software_update_urgent_svg_gz_len);
+                                    SOFTWARE_UPDATE_URGENT_SVG_GZ, SOFTWARE_UPDATE_URGENT_SVG_GZ_SIZE);
   AddTool(tb_follow, _("Follow"), m_followIcon,
           _("Return to the cell that is currently being evaluated"));
   EnableTool(tb_follow, false);
@@ -227,10 +242,10 @@ void ToolBar::AddTools()
 
   AddTool(tb_evaltillhere, _("Evaluate to point"), GetEvalTillHereBitmap(),
           _("Evaluate the file from its beginning to the cell above the cursor"));
-  
+
   AddTool(tb_evaluate_rest, _("Evaluate the rest"), GetEvalRestBitmap(),
           _("Evaluate the file from the cursor to its end"));
-  
+
 #ifndef __WXOSX__
   AddSeparator();
 #endif
@@ -265,11 +280,11 @@ void ToolBar::AddTools()
                             ));
   m_textStyle->SetSelection(textStyleSelection);
   AddControl(m_textStyle);
-  
+
   m_PlayButton = GetBitmap(wxT("media-playback-start"),
-                          media_playback_start_svg_gz,media_playback_start_svg_gz_len);
+                          MEDIA_PLAYBACK_START_SVG_GZ, MEDIA_PLAYBACK_START_SVG_GZ_SIZE);
   m_StopButton = GetBitmap(wxT("media-playback-stop"),
-                          media_playback_stop_svg_gz,media_playback_stop_svg_gz_len);
+                          MEDIA_PLAYBACK_STOP_SVG_GZ, MEDIA_PLAYBACK_STOP_SVG_GZ_SIZE);
 
   // It felt like a good idea to combine the play and the stop button.
   AddTool(tb_animation_startStop, _("Start or Stop animation"),
@@ -279,7 +294,7 @@ void ToolBar::AddTools()
 
 #if wxCHECK_VERSION(3, 1, 1)
   wxDisplay display;
-  
+
   int display_idx = wxDisplay::GetFromWindow(GetParent());
   if (display_idx < 0)
     m_ppi = wxSize(72,72);
@@ -305,19 +320,19 @@ void ToolBar::AddTools()
   m_plotSlider->SetToolTip(
     _("After clicking on animations created with with_slider_draw() or similar this slider allows to change the current frame."));
   m_plotSlider->Enable(false);
-  m_slideShowMaxIndex = -1;
-  m_slideShowDisplayedIndex = -1;
+  m_animationMaxIndex = -1;
+  m_animationDisplayedIndex = -1;
   AddControl(m_plotSlider);
   AddStretchSpacer(100);
   if(ShowHelp())
-    AddTool(wxID_HELP, _("Help"), GetHelpBitmap(), _("Show Maxima help"));
+    AddTool(wxID_HELP, _("Help"), GetHelpBitmap(), _("Show wxMaxima help"));
   Connect(wxEVT_SIZE,
           wxSizeEventHandler(ToolBar::OnSize),
           NULL, this);
   Connect(wxEVT_RIGHT_DOWN,
           wxMouseEventHandler(ToolBar::OnMouseRightDown),
           NULL, this);
-  Realize();  
+  Realize();
 }
 
 void ToolBar::UpdateBitmaps()
@@ -325,14 +340,14 @@ void ToolBar::UpdateBitmaps()
   wxSize ppi(-1,-1);
 #if wxCHECK_VERSION(3, 1, 1)
   wxDisplay display;
-  
+
   int display_idx = wxDisplay::GetFromWindow(GetParent());
   if (display_idx < 0)
     ppi = wxSize(72,72);
   else
     ppi = wxDisplay(display_idx).GetPPI();
 #endif
-  
+
   if((ppi.x <= 10) || (ppi.y <= 10))
     ppi = wxGetDisplayPPI();
 
@@ -353,12 +368,14 @@ void ToolBar::UpdateBitmaps()
     return;
 
   m_ppi = ppi;
-  
+
   SetToolBitmap(tb_eval,GetEvalBitmap());
   SetToolBitmap(tb_eval_all,GetEvalAllBitmap());
   SetToolBitmap(wxID_NEW,GetNewBitmap());
   SetToolBitmap(wxID_OPEN,GetOpenBitmap());
   SetToolBitmap(wxID_SAVE,GetSaveBitmap());
+  SetToolBitmap(wxID_UNDO,GetUndoBitmap());
+  SetToolBitmap(wxID_REDO,GetRedoBitmap());
   SetToolBitmap(wxID_PRINT,GetPrintBitmap());
   SetToolBitmap(wxID_PREFERENCES,GetPreferencesBitmap());
   SetToolBitmap(wxID_CUT,GetCutBitmap());
@@ -368,96 +385,104 @@ void ToolBar::UpdateBitmaps()
   SetToolBitmap(wxID_FIND,GetFindBitmap());
   SetToolBitmap(menu_restart_id,GetRestartBitmap());
   SetToolBitmap(tb_interrupt,GetInterruptBitmap());
-  m_followIcon = GetBitmap(wxT("weather-clear"),
-                          weather_clear_svg_gz,weather_clear_svg_gz_len);
+  m_followIcon = GetBitmap(wxT("arrow_up_square"),
+                          ARROW_UP_SQUARE_SVG_GZ, ARROW_UP_SQUARE_SVG_GZ_SIZE);
   m_needsInformationIcon = GetBitmap(wxT("software-update-urgent"),
-                                    software_update_urgent_svg_gz,software_update_urgent_svg_gz_len);
+                                    SOFTWARE_UPDATE_URGENT_SVG_GZ, SOFTWARE_UPDATE_URGENT_SVG_GZ_SIZE);
   SetToolBitmap(tb_follow, m_followIcon);
   SetToolBitmap(tb_evaltillhere, GetEvalTillHereBitmap());
   SetToolBitmap(tb_evaluate_rest, GetEvalRestBitmap());
   SetToolBitmap(tb_hideCode, GetHideCodeBitmap());
   m_PlayButton = GetBitmap(wxT("media-playback-start"),
-                           media_playback_start_svg_gz,media_playback_start_svg_gz_len);
+                           MEDIA_PLAYBACK_START_SVG_GZ, MEDIA_PLAYBACK_START_SVG_GZ_SIZE);
   m_StopButton = GetBitmap(wxT("media-playback-stop"),
-                           media_playback_stop_svg_gz,media_playback_stop_svg_gz_len);
+                           MEDIA_PLAYBACK_STOP_SVG_GZ, MEDIA_PLAYBACK_STOP_SVG_GZ_SIZE);
   SetToolBitmap(tb_animation_startStop,
                 m_PlayButton);
   SetToolBitmap(wxID_HELP,GetHelpBitmap());
   Realize();
 }
-  
+
 wxBitmap ToolBar::GetEvalAllBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("go-next"), go_jump_svg_gz, go_jump_svg_gz_len, siz);
+  return GetBitmap(wxT("go-next"), GO_JUMP_SVG_GZ, GO_JUMP_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetEvalBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("go-next"), go_next_svg_gz, go_next_svg_gz_len, siz);
+  return GetBitmap(wxT("go-next"), GO_NEXT_SVG_GZ, GO_NEXT_SVG_GZ_SIZE, siz);
 }
-wxBitmap ToolBar::GetNewBitmap(wxSize siz)  
+wxBitmap ToolBar::GetNewBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-new"),gtk_new_svg_gz,gtk_new_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-new"), GTK_NEW_SVG_GZ, GTK_NEW_SVG_GZ_SIZE, siz);
 }
 
 wxBitmap ToolBar::GetPrintBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-print"),gtk_print_svg_gz,gtk_print_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-print"), GTK_PRINT_SVG_GZ, GTK_PRINT_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetOpenBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-open"),gtk_open_svg_gz,gtk_open_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-open"), GTK_OPEN_SVG_GZ, GTK_OPEN_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetSaveBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-save"),gtk_save_svg_gz,gtk_save_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-save"), GTK_SAVE_SVG_GZ, GTK_SAVE_SVG_GZ_SIZE, siz);
+}
+wxBitmap ToolBar::GetUndoBitmap(wxSize siz)
+{
+  return GetBitmap(wxT("gtk-undo"), GTK_UNDO_SVG_GZ, GTK_UNDO_SVG_GZ_SIZE, siz);
+}
+wxBitmap ToolBar::GetRedoBitmap(wxSize siz)
+{
+  return GetBitmap(wxT("gtk-redo"), GTK_REDO_SVG_GZ, GTK_REDO_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetPreferencesBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-preferences"), gtk_preferences_svg_gz, gtk_preferences_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-preferences"), GTK_PREFERENCES_SVG_GZ, GTK_PREFERENCES_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetCutBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-cut"), gtk_cut_svg_gz, gtk_cut_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-cut"), GTK_CUT_SVG_GZ, GTK_CUT_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetCopyBitmap(wxSize siz)
 {
- return GetBitmap(wxT("gtk-copy"), gtk_copy_svg_gz, gtk_copy_svg_gz_len, siz);
+ return GetBitmap(wxT("gtk-copy"), GTK_COPY_SVG_GZ, GTK_COPY_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetPasteBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-paste"), gtk_paste_svg_gz, gtk_paste_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-paste"), GTK_PASTE_SVG_GZ, GTK_PASTE_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetSelectAllBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-select-all"), gtk_select_all_svg_gz,gtk_select_all_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-select-all"), GTK_SELECT_ALL_SVG_GZ, GTK_SELECT_ALL_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetFindBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-find"), gtk_find_svg_gz, gtk_find_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-find"), GTK_FIND_SVG_GZ, GTK_FIND_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetRestartBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("view-refresh"), view_refresh_svg_gz,view_refresh_svg_gz_len, siz);
+  return GetBitmap(wxT("view-refresh"), VIEW_REFRESH1_SVG_GZ, VIEW_REFRESH1_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetInterruptBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-stop"), gtk_stop_svg_gz, gtk_stop_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-stop"), GTK_STOP_SVG_GZ, GTK_STOP_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetEvalTillHereBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("go-bottom"), go_bottom_svg_gz, go_bottom_svg_gz_len, siz);
+  return GetBitmap(wxT("go-bottom"), GO_BOTTOM_SVG_GZ, GO_BOTTOM_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetHelpBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("gtk-help"), gtk_help_svg_gz,gtk_help_svg_gz_len, siz);
+  return GetBitmap(wxT("gtk-help"), GTK_HELP_SVG_GZ, GTK_HELP_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetEvalRestBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("go-last"), go_last_svg_gz,go_last_svg_gz_len, siz);
+  return GetBitmap(wxT("go-last"), GO_LAST_SVG_GZ, GO_LAST_SVG_GZ_SIZE, siz);
 }
 wxBitmap ToolBar::GetHideCodeBitmap(wxSize siz)
 {
-  return GetBitmap(wxT("weather-few-clouds"), weather_few_clouds_svg_gz,weather_few_clouds_svg_gz_len, siz);
+  return GetBitmap(wxT("eye-slash"), EYE_SLASH_SVG_GZ, EYE_SLASH_SVG_GZ_SIZE, siz);
 }
 
 void ToolBar::SetDefaultCellStyle()
@@ -475,7 +500,7 @@ void ToolBar::SetDefaultCellStyle()
     break;
   case 3:
     m_defaultCellStyle = GC_TYPE_SECTION;
-    break;    
+    break;
   case 4:
     m_defaultCellStyle = GC_TYPE_SUBSECTION;
     break;
@@ -596,8 +621,8 @@ void ToolBar::AnimationButtonState(AnimationStartStopState state)
         m_plotSlider->Enable(false);
         m_plotSlider->SetToolTip(
                 _("After clicking on animations created with with_slider_draw() or similar this slider allows to change the current frame."));
-        m_slideShowMaxIndex = -1;
-        m_slideShowDisplayedIndex = -1;
+        m_animationMaxIndex = -1;
+        m_animationDisplayedIndex = -1;
 
         if (m_AnimationStartStopState == Running)
         {
@@ -622,6 +647,9 @@ void ToolBar::OnMouseRightDown(wxMouseEvent &WXUNUSED(event))
   popupMenu->AppendCheckItem(shownew, _("New button"),
                              _("Show the \"New\" button?"));
   popupMenu->Check(shownew, ShowNew());
+  popupMenu->AppendCheckItem(undo_redo, _("Undo and redo button"),
+                             _("Show the undo and redo button?"));
+  popupMenu->Check(undo_redo, ShowUndoRedo());
   popupMenu->AppendCheckItem(open_save, _("Open and save button"),
                              _("Show the open and the save button?"));
   popupMenu->Check(open_save, ShowOpenSave());
@@ -666,6 +694,10 @@ void ToolBar::OnMenu(wxMenuEvent &event)
     ShowOpenSave(!ShowOpenSave());
     AddTools();
     break;
+  case undo_redo:
+    ShowUndoRedo(!ShowUndoRedo());
+    AddTools();
+    break;
   case print:
     ShowPrint(!ShowPrint());
     AddTools();
