@@ -46,6 +46,8 @@ public:
     varID_values,
     varID_functions,
     varID_arrays,
+    varID_macros,
+    varID_labels,
     varID_myoptions,
     varID_rules,
     varID_aliases,
@@ -63,7 +65,7 @@ public:
   explicit Variablespane(wxWindow *parent, wxWindowID id = wxID_ANY);
   //! Called when a variable name was changed
   void OnTextChange(wxGridEvent &event);
-  //! Called after te user has entered a variable name but before it is committed
+  //! Called after the user has entered a variable name but before it is committed
   void OnTextChanging(wxGridEvent &event);
   //! Called on right-clicking the variables list
   void OnRightClick(wxGridEvent &event);
