@@ -33,12 +33,13 @@
 #include "stx/optional.hpp"
 #include "precomp.h"
 #include <wx/wx.h>
+#include <wx/xml/xml.h>
 #include <wx/aui/aui.h>
 #include <wx/textfile.h>
 #include <wx/fdrepdlg.h>
 #include <wx/dc.h>
+#include <thread>
 #include <list>
-
 #include "CellPointers.h"
 #include "VariablesPane.h"
 #include "Notification.h"
@@ -73,7 +74,7 @@ a big bitmap) is provided by wxWidgets:
  - and the RefreshRect() method notifies wxWidgets that a rectangular region
    contains changes that need to be redrawn.
 
-The worksheet isn't immediately redrawn on a key press, a mouse klick or on
+The worksheet isn't immediately redrawn on a key press, a mouse click or on
 maxima outputting new data. Instead all such events are processed in order until
 wxMaxima has caught up with all the events (which causes wxWidgets to send a Idle
 event to the wxMaxima object) or until we reach a timeout. If the user manages
@@ -89,6 +90,9 @@ public:
   //! Is an update of the worksheet controls needed?
   bool UpdateControlsNeeded(){bool result = m_updateControls; m_updateControls = false; return result;}
 private:
+  wxString m_maximaVersion;
+  //! The directory with maxima's documentation
+  wxString m_maximaDocDir;
   //! The storage for UpdateControlsNeeded()
   bool m_updateControls = true;
   //! Is a scroll to the cursor scheduled?
@@ -99,7 +103,7 @@ private:
   int m_newxPosition;
   // The y position to scroll to
   int m_newyPosition;
-  // false = collect scroll events without redrawing for every single one  
+  // false = collect scroll events without redrawing for every single one
   bool m_dontSkipScrollEvent;
   //! Which zoom level were we at when we started the zoom gesture?
   double m_zoomAtGestureStart;
@@ -109,8 +113,8 @@ private:
   bool m_windowActive;
   //! The configuration storage
   Configuration m_configurationTopInstance;
-  //! The rectangle we need to refresh. -1 as "left" coordinate means: No rectangle
-  wxRect m_rectToRefresh;
+  //! The rectangle we need to refresh.
+  wxRegion m_rectToRefresh;
   /*! The size of a scroll step
 
     Defines the size of a
@@ -209,7 +213,7 @@ private:
   /*! \defgroup UndoBufferFill Undo methods for cell additions/deletions:
 
     Each EditorCell has its own private undo buffer Additionally wxMaxima
-    maintains a undo buffer for worksheet changes. It works the following way:
+    maintains an undo buffer for worksheet changes. It works the following way:
 
      - Cells normally aren't deleted. They are moved into an undo buffer instead.
      - The undo buffer is also notified when Cells that are added
@@ -467,7 +471,7 @@ private:
   void OnMouseRightDown(wxMouseEvent &event);
 
   void OnSidebarKey(wxCommandEvent &event);
-  
+
   void OnMouseLeftUp(wxMouseEvent &event);
 
   //! Is called if we loose the mouse connection whilst selecting text/cells
@@ -511,7 +515,7 @@ private:
    * depending on where we first clicked. If m_clickType equals
    * CLICK_TYPE_NONE - click-dragging does not result in a selection (we clicked in hideRect for instance)
    * CLICK_TYPE_GROUP_SELECTION - we are selecting full groupcells only. Only y-coordinate matters.
-   * CLICK_TYPE_INPUT_SELECTION - we clicked in an editor (GroupCell::GetEditable()) and draging
+   * CLICK_TYPE_INPUT_SELECTION - we clicked in an editor (GroupCell::GetEditable()) and dragging
    *   results in selecting text in EditorCell
    * CLICK_TYPE_OUTPUT_SELECTION - we clicked in an output, we want selection to be confined to that
    *   GroupCell's output. GC we first clicked in was stored in OnMouseMotion method
@@ -539,7 +543,7 @@ private:
   //! Finds the end of the current chapter/section/...
   GroupCell *EndOfSectioningUnit(GroupCell *start);
 
-  //! Is called if a action from the autocomplete menu is selected
+  //! Is called if an action from the autocomplete menu is selected
   void OnComplete(wxCommandEvent &event);
 
   /*! Is called if wxWidgets wants to erase the worksheet's background
@@ -602,6 +606,9 @@ private:
   AutocompletePopup *m_autocompletePopup;
 
 public:
+  wxString GetMaximaHelpFile();
+  void SetMaximaVersion(wxString version){m_maximaVersion = version;}
+  wxString GetMaximaVersion(){return m_maximaVersion;}
   //! Is this worksheet empty?
   bool IsEmpty() const
   { return !m_tree || (!m_tree->GetNext() && m_tree->GetEditable()->GetValue().Length()<=1); }
@@ -630,7 +637,11 @@ public:
   //! Renumber all sections
   void NumberSections();
 
-  //! A error notification message
+  //! Make this chapter/section/... a section/subsection/... changing its subheadings, too.
+  bool SectioningMoveIn();
+  //! Make this section/subsection/... a chapter/section/... changing its subheadings, too.
+  bool SectioningMoveOut();
+//! A error notification message
   stx::optional<Notification> m_notificationMessage;
   //! Is this window active?
   void WindowActive(bool active){m_windowActive = active;}
@@ -747,7 +758,7 @@ public:
 
   //! Is a Redraw requested?
   bool RedrawRequested() const
-  { return m_redrawRequested || m_mouseMotionWas || m_rectToRefresh.GetLeft() != -1; }
+    { return m_redrawRequested || m_mouseMotionWas || m_rectToRefresh.IsEmpty(); }
 
   //! To be called after enabling or disabling the visibility of code cells
   void CodeCellVisibilityChanged();
@@ -755,6 +766,9 @@ public:
   //! Re-read the configuration
   void UpdateConfig() { m_configuration->ReadConfig(); }
 
+  //! The thread the help file anchors are compiled in
+  std::unique_ptr<std::thread> m_helpfileanchorsThread;
+
   //! The name of the currently-opened file
   wxString m_currentFile;
 
@@ -859,7 +873,9 @@ public:
     popid_digits_50,
     popid_digits_100,
     popid_digits_all,
+    popid_digits_all_linebreak,
     popid_image,
+    popid_change_image,
     popid_svg,
     popid_emf,
     popid_animation_save,
@@ -880,6 +896,8 @@ public:
     popid_fold,
     popid_unfold,
     popid_maxsizechooser,
+    popid_resolutionchooser,
+    popid_reloadimage,
     popid_suggestion1,
     popid_suggestion2,
     popid_suggestion3,
@@ -891,7 +909,44 @@ public:
     popid_suggestion9,
     popid_suggestion10,
     popid_hide_tooltipMarker,
-    popid_hide_tooltipMarkerForThisMessage
+    popid_hide_tooltipMarkerForThisMessage,
+    popid_property_real,
+    popid_property_imaginary,
+    popid_property_complex,
+    popid_property_additive,
+    popid_property_alphabetic,
+    popid_property_bindtest,
+    popid_property_antisymmetric,
+    popid_property_commutative,
+    popid_property_symmetric,
+    popid_property_constant,
+    popid_property_even,
+    popid_property_odd,
+    popid_property_evenfun,
+    popid_property_oddfun,
+    popid_property_increasing,
+    popid_property_decreasing,
+    popid_property_integer,
+    popid_property_noninteger,
+    popid_property_integervalued,
+    popid_property_lassociative,
+    popid_property_rassociative,
+    popid_property_linear,
+    popid_property_mainvar,
+    popid_property_multiplicative,
+    popid_property_nary,
+    popid_property_nonarray,
+    popid_property_nonscalar,
+    popid_property_scalar,
+    popid_property_noun,
+    popid_property_outative,
+    popid_property_posfun,
+    popid_property_rational,
+    popid_property_irrational,
+    popid_property_greaterThan,
+    popid_property_evfun,
+    popid_property_evflag,
+    popid_property_atvalue
   };
 
   //! The constructor
@@ -1016,6 +1071,8 @@ public:
   */
   void DeleteSelection();
 
+  void TOCdnd();
+  
   //! Is it possible to delete the cells between start and end?
   bool CanDeleteRegion(GroupCell *start, GroupCell *end) const;
 
@@ -1046,7 +1103,7 @@ public:
   void DivideCell();
 
   void MergeCells();
-  
+
   void SetLastQuestion(const wxString &lastQuestion){m_lastQuestion = lastQuestion;}
   wxString GetLastQuestion(){return m_lastQuestion;}
 
@@ -1147,6 +1204,7 @@ public:
   wxString GetString(bool lb = false);
 
   GroupCell *GetTree() const { return m_tree.get(); }
+  std::unique_ptr<GroupCell> *GetTreeAddress() { return &m_tree; }
 
   /*! Return the first of the currently selected cells.
 
@@ -1177,7 +1235,32 @@ public:
 
   bool ActivatePrevInput() { return ActivateInput(-1); }
   bool ActivateNextInput() { return ActivateInput(+1); }
+  wxString GetStatusText() const{return m_statusText;}
+  bool StatusTextChangedHas(){
+    bool retval = (m_statusTextHas != m_statusTextHas_old) ||
+      (m_statusText != m_statusText_old);
+    m_statusText_old = m_statusText;
+    m_statusTextHas_old = m_statusTextHas;
+    return retval;
+  }
+  bool StatusTextHas(){return m_statusTextHas;}
 private:
+  template <class T_SRC, class T_DEST>
+  inline std::unique_ptr<T_DEST> unique_cast(std::unique_ptr<T_SRC> &&src)
+    {
+      if (!src) return std::unique_ptr<T_DEST>();
+
+      T_DEST *dest_ptr = &dynamic_cast<T_DEST &>(*src.get());
+
+      src.release();
+      return std::move(std::unique_ptr<T_DEST>(dest_ptr));
+    }
+  wxString m_statusText;
+  wxString m_statusText_old;
+  bool m_statusTextHas = false;
+  bool m_statusTextHas_old = false;
+  void StatusText(wxString text){m_statusText = text;m_statusTextHas = true;}
+  void UnsetStatusText(){m_statusTextHas = false;}
   bool ActivateInput(int direction);
 
 public:
@@ -1193,7 +1276,7 @@ public:
 
   //! Scrolls to the cell given by ScheduleScrollToCell; Is called once we have time to do so.
   void ScrollToCellIfNeeded();
-  
+
   //! Schedules scrolling to a given cell
   void ScheduleScrollToCell(Cell *cell, bool scrollToTop = true)
     {
@@ -1237,7 +1320,7 @@ public:
   //! Does the selection start with a cell of the type "type"
   bool IsSelected(CellType type);
 
-  /*! Set the slide of the currently selected slideshow or advance it by one step
+  /*! Set the slide of the currently selected animation or advance it by one step
 
     \param change
       - >=0: The slide the animation has to be set to
@@ -1266,7 +1349,7 @@ public:
   void SetDefaultHCaret();
 
   /*! Set the HCaret at the location of the given Cell.
-    
+
     @param where   The cell to place the cursor before.
   */
   void SetHCaret(GroupCell *where); // call with false, when manually refreshing
@@ -1335,7 +1418,7 @@ public:
   { return m_saved; }
 
   void SetSaved(bool saved)
-  { m_saved = saved; }
+    { if(m_saved != saved) m_updateControls = true;m_saved = saved;}
 
   void OutputChanged()
     {
@@ -1412,10 +1495,10 @@ public:
 
     Why we need to override this signal is that on Linux and at least on
     wxGTK and wxWidgets 3.0+3.1 each single scroll step generates its own
-    thumbtrack event that forces an immediate scroll of the worksheet 
-    including an immediate redraw. It therefore is possible for an user 
-    to queue hundreds of thumbtrack events per seconds while wxMaxima's 
-    framerate (and event processing speed) might be considerably lower 
+    thumbtrack event that forces an immediate scroll of the worksheet
+    including an immediate redraw. It therefore is possible for a user
+    to queue hundreds of thumbtrack events per seconds while wxMaxima's
+    framerate (and event processing speed) might be considerably lower
     than that.
 
     OnThumbtrack is a try to merge all scroll events until the idle task
@@ -1459,8 +1542,17 @@ public:
 
   wxString GetOutputAboveCaret();
 
-  void LoadSymbols()
-  { m_autocomplete.LoadSymbols(); }
+  void LoadSymbols();
+  //! Collect all keyword anchors in the help file
+  void CompileHelpFileAnchors();
+  //! Load the result from the last CompileHelpFileAnchors from the disk cache
+  bool LoadManualAnchorsFromCache();
+  //! Load the help file anchors from an wxXmlDocument
+  bool LoadManualAnchorsFromXML(wxXmlDocument xmlDocument, bool checkManualVersion = true);
+  //! Load the help file anchors from the built-in list
+  bool LoadBuiltInManualAnchors();
+  //! Save the list of help file anchors to the cache.
+  void SaveManualAnchorsToCache();
 
   bool Autocomplete(AutoComplete::autoCompletionType type = AutoComplete::command);
 
@@ -1515,6 +1607,11 @@ public:
   void QuestionPending(bool pending)
   { m_questionPrompt = pending; }
 
+  void SetMaximaDocDir(wxString dir)
+    {
+      m_maximaDocDir = dir;
+    }
+
 //! Does the GroupCell cell points to contain the question currently asked by maxima?
   bool GCContainsCurrentQuestion(GroupCell *cell);
 
@@ -1539,8 +1636,6 @@ public:
 
   //! All anchors for keywords maxima's helpfile contains
   HelpFileAnchors m_helpFileAnchors;
-  //! Is the help file anchors available
-  bool m_helpFileAnchorsUsable;
   //! Suggestions for how the word that was right-clicked on could continue
   wxArrayString m_replacementsForCurrentWord;
   //Simple iterator over a Maxima input string, skipping comments and strings
@@ -1548,13 +1643,13 @@ public:
   {
   public:
     explicit SimpleMathConfigurationIterator(const wxString &ainput);
-    
+
     void operator++();
 
     bool isValid() const
       { return pos < input.length(); }
-    
-    
+
+
     inline wxChar operator*() const
       { return input[pos]; }
 
@@ -1566,7 +1661,7 @@ public:
      */
     const wxString &input;
   };
-    
+
 #if wxUSE_ACCESSIBILITY
   class AccessibilityInfo: public wxAccessible
   {
@@ -1610,7 +1705,11 @@ protected:
 };
 
 inline Worksheet *Cell::GetWorksheet() const
-{ return static_cast<Worksheet*>((*m_configuration)->GetWorkSheet()); }
+{
+  wxWindow *worksheet = (*m_configuration)->GetWorkSheet();
+  wxASSERT(worksheet != NULL);
+  return static_cast<Worksheet*>(worksheet);
+}
 
 inline void Configuration::SetWorkSheet(wxWindow *workSheet)
 { m_workSheet = dynamic_cast<Worksheet*>(workSheet); }
