@@ -59,10 +59,10 @@ const wxString &Cell::GetToolTip(const wxPoint point) const
 }
 
 Cell::Cell(GroupCell *group, Configuration **config) :
-    m_group(group),
-    m_configuration(config),
-    m_toolTip(&wxm::emptyString),
-    m_fontSize_Scaled(Scale_Px((*config)->GetMathFontSize()))
+  m_group(group),
+  m_configuration(config),
+  m_toolTip(&wxm::emptyString),
+  m_fontSize_Scaled(-1)
 {
   wxASSERT(group && (group->GetType() == MC_TYPE_GROUP || group == this));
   InitBitFields();
@@ -158,17 +158,17 @@ void Cell::CopyCommonData(const Cell & cell)
     BreakUp();
 }
 
-std::unique_ptr<Cell> Cell::CopyList() const
+std::unique_ptr<Cell> Cell::CopyList(GroupCell *group) const
 {
   CellListBuilder<> copy;
   for (auto &src : OnList(this))
-    copy.Append(src.Copy());
+    copy.Append(src.Copy(group));
   return std::move(copy);
 }
 
-std::unique_ptr<Cell> Cell::CopyList(const Cell *cell)
+std::unique_ptr<Cell> Cell::CopyList(GroupCell *group, const Cell *cell)
 {
-  return cell ? cell->CopyList() : nullptr;
+  return cell ? cell->CopyList(group) : nullptr;
 }
 
 void Cell::ClearCacheList()
@@ -218,9 +218,11 @@ GroupCell *Cell::GetGroup() const
 
 bool Cell::NeedsRecalculation(AFontSize fontSize) const
 {
-  bool const result = (!HasValidSize()) ||
-                      (GetType() != MC_TYPE_GROUP && !EqualToWithin(Scale_Px(fontSize), m_fontSize_Scaled, 0.1f));
-  return result;
+  if (!HasValidSize())
+    return true;
+  if (GetType() == MC_TYPE_GROUP)
+    return false;
+  return !EqualToWithin(Scale_Px(fontSize), m_fontSize_Scaled, 0.1f);
 }
 
 int Cell::GetCenterList() const
@@ -428,6 +430,9 @@ bool Cell::DrawThisCell(wxPoint point)
   if(m_isBrokenIntoLines)
     return false;
   
+  if((*m_configuration)->GetPrinting())
+    return true;
+
   if(!(*m_configuration)->ClipToDrawRegion())
     return true;
   
@@ -437,12 +442,12 @@ bool Cell::DrawThisCell(wxPoint point)
 bool Cell::HasValidSize() const
 {
   return !m_recalculateWidths &&
-         m_width > 0 && m_height > 0 && m_center >= 0;
+         m_width >= 0 && m_height >= 0 && m_center >= 0;
 }
 
 bool Cell::HasStaleSize() const
 {
-  return m_width > 0 && m_height > 0 && m_center >= 0;
+  return m_width >= 0 && m_height >= 0 && m_center >= 0;
 }
 
 bool Cell::HasValidPosition() const
@@ -450,6 +455,19 @@ bool Cell::HasValidPosition() const
   return (m_currentPoint.x >= 0) & (m_currentPoint.y >= 0);
 }
 
+void Cell::SetConfigurationList(Configuration **config)
+{
+  for (Cell &tmp : OnList(this))
+    tmp.SetConfiguration(config);
+}
+
+void Cell::SetConfiguration(Configuration **config)
+{
+  m_configuration = config;
+  for (Cell &cell : OnInner(this))
+    cell.SetConfigurationList(config);
+}
+
 wxRect Cell::GetRect(bool wholeList) const
 {
   if (wholeList)
@@ -531,7 +549,7 @@ wxString Cell::ListToString() const
   wxString retval;
   bool firstline = true;
 
-  for (const Cell &tmp : OnDrawList(this))
+  for (const Cell &tmp : OnList(this))
   {
     if ((!firstline) && (tmp.m_forceBreakLine))
     {
@@ -929,11 +947,15 @@ wxString Cell::ListToXML() const
 }
 
 /***
- * Get the part for diff tag support - only ExpTag overvrides this.
+ * Get the part for diff tag support - only ExptCell and SubSupCell override this.
  */
 wxString Cell::GetDiffPart() const
 {
-  return {};
+  wxString s = ToString();
+  if (s == wxEmptyString)
+    return s;
+
+  return wxT(",") + s + wxT(",1");
 }
 
 Cell::Range Cell::GetCellsInRect(const wxRect &rect) const
@@ -1060,32 +1082,25 @@ Cell *Cell::last() const
 
 bool Cell::BreakUp()
 {
-  bool retval = false;
-  int clientWidth = .8*(*m_configuration)->GetClientWidth() - (*m_configuration)->GetIndent();
-  if(clientWidth < 50)
-    clientWidth = 50;
-  for (Cell &cell : OnInner(this))
-    for (Cell &tmp : OnList(&cell))
-      if(tmp.GetWidth() > clientWidth)
-      {
-        tmp.BreakUp();
-        retval = true;
-    }
-  return retval;
+  return false;
 }
 
 void Cell::BreakUpAndMark()
 {
   wxASSERT(!m_isBrokenIntoLines);
   Cell::BreakUp();
+  if(!m_isBrokenIntoLines)
+    m_recalculateWidths = true;
   m_isBrokenIntoLines = true;
-  m_recalculateWidths = true;
 }
 
 void Cell::Unbreak()
 {
   if(m_isBrokenIntoLines)
+  {
     ResetData();
+    m_recalculateWidths = true;
+  }
 
   m_isBrokenIntoLines = false;
   SetNextToDraw(m_next);
