@@ -48,12 +48,13 @@
 class ConjugateCell final : public Cell
 {
 public:
-  ConjugateCell(GroupCell *parent, Configuration **config, std::unique_ptr<Cell> &&inner);
-  ConjugateCell(const ConjugateCell &cell);
-  std::unique_ptr<Cell> Copy() const override;
+  ConjugateCell(GroupCell *group, Configuration **config, std::unique_ptr<Cell> &&inner);
+  ConjugateCell(GroupCell *group, const ConjugateCell &cell);
+  std::unique_ptr<Cell> Copy(GroupCell *group) const override;
   const CellTypeInfo &GetInfo() override;
 
   int GetInnerCellCount() const override { return 3; }
+  // cppcheck-suppress objectIndex
   Cell *GetInnerCell(int index) const override { return (&m_open)[index].get(); }
 
   bool BreakUp() override;
@@ -74,7 +75,7 @@ private:
 //** Bitfield objects (0 bytes)
 //**
   void InitBitFields()
-  { // Keep the initailization order below same as the order
+  { // Keep the initialization order below same as the order
     // of bit fields in this class!
   }
 
