@@ -67,10 +67,11 @@ private:
 
 public:
   //! The constructor
-  EditorCell(GroupCell *parent, Configuration **config, const wxString &text = {});
-  EditorCell(const EditorCell &cell);
-  std::unique_ptr<Cell> Copy() const override;
+  EditorCell(GroupCell *group, Configuration **config, const wxString &text = {});
+  EditorCell(GroupCell *group, const EditorCell &cell);
+//  std::unique_ptr<Cell> Copy(GroupCell *group) const override;
   const CellTypeInfo &GetInfo() override;
+  std::unique_ptr<Cell> Copy(GroupCell *group) const override;
 
   //! Get the previous EditorCell in the list
   EditorCell *GetPrevious() const { return dynamic_cast<EditorCell*>(Cell::GetPrevious()); }
@@ -205,8 +206,6 @@ public:
   //! Determines which index the char at the position "x chars left, y chars down" is at.
   int XYToPosition(int x, int y);
 
-  void SetCurrentPoint(wxPoint point) override;
-
   //! The screen coordinates of the cursor
   wxPoint PositionToPoint(int pos = -1) override;
 
@@ -402,6 +401,9 @@ public:
   //! Convert the current selection to a string
   wxString GetSelectionString() const;
 
+  //! Try to determine the selection's text style
+  TextStyle GetSelectionStyle() const;
+
   //! The word the cursor currently is at
   wxString GetWordUnderCaret();
 
@@ -455,6 +457,8 @@ public:
   void SetType(CellType type) override;
   void SetStyle(TextStyle style) override;
 
+  bool NeedsRecalculation(AFontSize fontSize) const override;
+
   //! Return to the selection after the cell has been left downwards
   void ReturnToSelectionFromBot()
   {
@@ -462,7 +466,13 @@ public:
   }
 
   //! Get the list of commands, parenthesis, strings and whitespaces in a code cell
-  const MaximaTokenizer::TokenList &GetTokens() const {return m_tokens;}
+  const MaximaTokenizer::TokenList &GetTokens()
+    {
+      // If we never show a code cell it might still not be tokenized
+      if(!(*m_configuration)->ShowCodeCells())
+        StyleText();
+      return m_tokens; 
+    }
 
 private:
   //! Did the zoom factor change since the last recalculation?
@@ -524,7 +534,7 @@ private:
     //! If IsStyleSet() is true this function returns the style of this text
     //! portion
     TextStyle GetStyle() const { return m_style; }
-    // Has a individual text style been set for this text portion?
+    // Has an individual text style been set for this text portion?
     bool IsStyleSet() const { return m_styleThisText; }
   };
 
@@ -559,7 +569,7 @@ private:
 
   /*! Handle ESC shortcuts for special characters
 
-    These characters can be tought to LaTeX and the html browser if necessary in
+    These characters can be thought to LaTeX and the html browser if necessary in
     TextCell::ToTeX and EditorCell::ToTeX. They can also be
     converted to maxima strings in wxMaxima::SendMaxima.
    */
@@ -603,10 +613,6 @@ private:
 
   std::vector<HistoryEntry> m_history;
 
-//** 8 bytes
-//**
-  wxPoint m_currentPoint_Last{-1, -1};
-
 //** 8/4 bytes
 //**
   AFontName m_fontName;
@@ -656,7 +662,7 @@ private:
 //** Bitfield objects (2 bytes)
 //**
   void InitBitFields()
-  { // Keep the initailization order below same as the order
+  { // Keep the initialization order below same as the order
     // of bit fields in this class!
     m_autoAnswer = false;
     m_containsChanges = false;
