@@ -36,7 +36,7 @@
 #include "CellPointers.h"
 #include "ImgCell.h"
 #include "MarkDown.h"
-#include "SlideShowCell.h"
+#include "AnimationCell.h"
 #include "TextCell.h"
 #include "LabelCell.h"
 #include "stx/unique_cast.hpp"
@@ -129,92 +129,95 @@ GroupCell::GroupCell(Configuration **config, GroupType groupType, const wxString
     m_inputLabel->SetType(MC_TYPE_MAIN_PROMPT);
   }
 
-  std::unique_ptr<EditorCell> editor;
-  switch (groupType)
-  {
-    case GC_TYPE_CODE:
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_INPUT);
-      break;
-    case GC_TYPE_TEXT:
-      m_inputLabel->SetType(MC_TYPE_TEXT);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_TEXT);
-      break;
-    case GC_TYPE_TITLE:
-      m_inputLabel->SetType(MC_TYPE_TITLE);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_TITLE);
-      break;
-    case GC_TYPE_SECTION:
-      m_inputLabel->SetType(MC_TYPE_SECTION);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_SECTION);
-      break;
-    case GC_TYPE_SUBSECTION:
-      m_inputLabel->SetType(MC_TYPE_SUBSECTION);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_SUBSECTION);
-      break;
-    case GC_TYPE_SUBSUBSECTION:
-      m_inputLabel->SetType(MC_TYPE_SUBSUBSECTION);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_SUBSUBSECTION);
-      break;
-    case GC_TYPE_HEADING5:
-      m_inputLabel->SetType(MC_TYPE_HEADING5);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_HEADING5);
-      break;
-    case GC_TYPE_HEADING6:
-      m_inputLabel->SetType(MC_TYPE_HEADING6);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_HEADING6);
-      break;
-    case GC_TYPE_IMAGE:
-      m_inputLabel->SetType(MC_TYPE_IMAGE);
-      editor = std::make_unique<EditorCell>(this, m_configuration);
-      editor->SetType(MC_TYPE_IMAGE);
-      break;
-    default:
-      break;
-  }
+  std::unique_ptr<EditorCell> editor = std::make_unique<EditorCell>(this, m_configuration);
 
   if (editor && !initString.empty())
     editor->SetValue(initString);
   if (editor)
     AppendInput(std::move(editor));
 
+  SetGroupType(groupType);
+    
+
   // when creating an image cell, if a string is provided
   // it loads an image (without deleting it)
   if ((groupType == GC_TYPE_IMAGE) && (initString.Length() > 0))
   {
     std::unique_ptr<Cell> ic;
-    if (wxImage::GetImageCount(initString) < 2)
+    if (wxImage::GetImageCount(initString) < 2) {
       ic = std::make_unique<ImgCell>(this, m_configuration, initString, std::shared_ptr<wxFileSystem>{} /* system fs */, false);
+
+      // Since this is the (only?) place where an ImgCell is constructed when the user manually
+      // inserts an image file (not loaded from zip or gnuplot tempfile), set the filename such that reloading the
+      // file later is possible.
+      static_cast<ImgCell &>(*ic).SetOrigImageFile(initString);
+    }
     else
-      ic = std::make_unique<SlideShow>(this, m_configuration, initString, false);
+      ic = std::make_unique<AnimationCell>(this, m_configuration, initString, false);
     AppendOutput(std::move(ic));
   }
 }
 
-GroupCell::GroupCell(const GroupCell &cell):
-    GroupCell(cell.m_configuration, cell.m_groupType)
+GroupCell::GroupCell(GroupCell *WXUNUSED(group), GroupCell const &cell):
+  GroupCell::GroupCell(cell)
+{}
+
+GroupCell::GroupCell(GroupCell const &cell):
+  GroupCell(cell.m_configuration, cell.m_groupType)
 {
   CopyCommonData(cell);
   if (cell.m_inputLabel)
-    SetInput(cell.m_inputLabel->CopyList());
+    SetInput(cell.m_inputLabel->CopyList(this));
   if (cell.m_output)
-    SetOutput(cell.m_output->CopyList());
+    SetOutput(cell.m_output->CopyList(this));
   SetAutoAnswer(cell.m_autoAnswer);
-  UpdateYPosition();
 }
 
 DEFINE_CELL(GroupCell)
 
+void GroupCell::SetGroupType(GroupType groupType)
+{
+  CellType type = MC_TYPE_DEFAULT;
+  switch (groupType)
+  {
+    case GC_TYPE_CODE:
+      type = MC_TYPE_INPUT; break;
+    case GC_TYPE_TEXT:
+      type = MC_TYPE_TEXT; break;
+    case GC_TYPE_TITLE:
+      type = MC_TYPE_TITLE; break;
+    case GC_TYPE_SECTION:
+      type = MC_TYPE_SECTION; break;
+    case GC_TYPE_SUBSECTION:
+      type = MC_TYPE_SUBSECTION; break;
+    case GC_TYPE_SUBSUBSECTION:
+      type = MC_TYPE_SUBSUBSECTION; break;
+    case GC_TYPE_HEADING5:
+      type = MC_TYPE_HEADING5; break;
+    case GC_TYPE_HEADING6:
+      type = MC_TYPE_HEADING6; break;
+    case GC_TYPE_IMAGE:
+      type = MC_TYPE_IMAGE; break;
+    default:
+      break;
+  }
+  if(GetEditable())
+    GetEditable()->SetType(type);
+  if(GetPrompt())
+  {
+    if(groupType == GC_TYPE_CODE)
+      GetPrompt()->SetType(MC_TYPE_MAIN_PROMPT);
+    else
+      GetPrompt()->SetType(type);
+  }
+  m_groupType = groupType;
+  ResetSize();
+}
+
 std::unique_ptr<GroupCell> GroupCell::CopyList() const
 {
-  auto copy = Cell::CopyList();
+  GroupCell parent(m_configuration, GC_TYPE_CODE);
+  auto copy = Cell::CopyList(&parent);
   return stx::static_unique_ptr_cast<GroupCell>(std::move(copy));
 }
 
@@ -353,6 +356,8 @@ void GroupCell::SetOutput(std::unique_ptr<Cell> &&output)
 
 void GroupCell::RemoveOutput()
 {
+  if(!m_output)
+    return;
   m_numberedAnswersCount = 0;
   if (m_output == NULL)
     return;
@@ -376,7 +381,7 @@ void GroupCell::RemoveOutput()
     m_width = 50;
   (*m_configuration)->AdjustWorksheetSize(true);
   
-  // Move all cells that follow the current one up by the amount this cell has shrinked.
+  // Move all cells that follow the current one up by the amount this cell has shrunk.
   UpdateCellsInGroup();
 
   m_updateConfusableCharWarnings = true;
@@ -396,20 +401,11 @@ void GroupCell::AppendOutput(std::unique_ptr<Cell> &&cell)
       input->ContainsChanges(false);
   }
   else
-  {
     CellList::AppendCell(m_output, std::move(cell));
-  }
+
   UpdateCellsInGroup();
   m_updateConfusableCharWarnings = true;
-  ResetSize();
-
-  // IMPORTANT! Do not Recalculate() here, as it is quadratic behavior
-  // and it makes interactive performance go down the drain. If there are
-  // visual glitches that a recalculation here would "fix", they must be
-  // fixed at their source. Doing it by recalculating here will only patch
-  // around the problem and make wxMaxima perform like molasses. IMPORTANT!
-  if (TEMPORARY_WINDOWS_PERFORMANCE_HACK)
-    Recalculate(); // FIXME this has to go away eventually
+  m_cellsAppended = true;
 }
 
 WX_DECLARE_STRING_HASH_MAP(int, CmdsAndVariables);
@@ -489,6 +485,8 @@ void GroupCell::Recalculate()
       m_width = configuration->GetCellBracketWidth();
       m_height = 2;
       m_center = 1;
+      m_clientWidth_old = (*m_configuration)->GetClientWidth();
+      m_cellsAppended = false;
       return;
     }
     
@@ -496,12 +494,19 @@ void GroupCell::Recalculate()
       RecalculateInput();
 
     RecalculateOutput();
+    m_height = m_outputRect.GetHeight() + m_inputHeight;
+    // Move all cells that follow the current one down by the amount this cell has grown.
+    (*m_configuration)->AdjustWorksheetSize(true);
+    
+    m_cellsAppended = false;
+    m_clientWidth_old = (*m_configuration)->GetClientWidth();
   }
   // The line breaking will have set our "needs recalculation" flag again.
   UpdateYPosition();
-  Cell::Recalculate((*m_configuration)->GetDefaultFontSize(
-                      ));
+  Cell::Recalculate((*m_configuration)->GetDefaultFontSize());
+  m_cellsAppended = false;
   m_clientWidth_old = (*m_configuration)->GetClientWidth();
+  wxASSERT(!NeedsRecalculation((*m_configuration)->GetDefaultFontSize()));
 }
 
 void GroupCell::InputHeightChanged()
@@ -527,8 +532,7 @@ void GroupCell::InputHeightChanged()
 // breakup cells and compute new line breaks
 void GroupCell::OnSize()
 {
-  InputHeightChanged();
-  BreakLines();
+  Recalculate();
 }
 
 AFontSize GroupCell::EditorFontSize() const
@@ -562,6 +566,7 @@ void GroupCell::RecalculateInput()
   // special case
   if (m_groupType == GC_TYPE_PAGEBREAK)
   {
+    m_height = Scale_Px(3);
     m_inputWidth = m_width = configuration->GetCellBracketWidth();
     m_inputHeight = m_height = 2;
     m_center = 0;
@@ -582,11 +587,17 @@ void GroupCell::RecalculateInput()
         m_inputLabel->Recalculate(EditorFontSize());
         m_inputWidth = m_width = m_inputLabel->GetFullWidth();
         m_center = m_inputLabel->GetCenterList();
+        m_inputLabel->ResetCellListSizesList();
         m_inputHeight = m_height = m_inputLabel->GetHeightList();
       }
+      m_height = m_inputHeight;
+    }
+    else
+    {
+      m_inputHeight = 0;
+      m_height = 0;
     }
   }
-  m_height = m_inputHeight;
   if(!m_inputLabel)
     m_center = m_height;
   m_width = m_inputWidth;
@@ -655,17 +666,13 @@ void GroupCell::RecalculateOutput()
         m_outputRect.height += MC_LINE_SKIP;
     }
   }
-
-  m_height = m_outputRect.GetHeight() + m_inputHeight;
-  // Move all cells that follow the current one down by the amount this cell has grown.
-  (*m_configuration)->AdjustWorksheetSize(true);
 }
 
 bool GroupCell::NeedsRecalculation(AFontSize fontSize) const
 {
   return Cell::NeedsRecalculation(fontSize) ||
-    (GetEditable() && GetEditable()->NeedsRecalculation(EditorFontSize())) ||
-    (m_clientWidth_old != (*m_configuration)->GetClientWidth());
+//    (GetEditable() && GetEditable()->NeedsRecalculation(EditorFontSize())) ||
+    (m_clientWidth_old != (*m_configuration)->GetClientWidth()) || m_cellsAppended;
 }
 
 void GroupCell::UpdateYPositionList()
@@ -694,6 +701,15 @@ void GroupCell::UpdateYPosition()
                  previous->GetMaxDrop();
   }
   m_currentPoint = point;
+
+  EditorCell *editor = GetEditable();
+  if (editor)
+    editor->SetCurrentPoint(CalculateInputPosition());
+}
+
+wxPoint GroupCell::CalculateInputPosition()
+{
+  return wxPoint(m_currentPoint.x + GetInputIndent(), m_currentPoint.y);
 }
 
 int GroupCell::GetInputIndent()
@@ -715,22 +731,49 @@ int GroupCell::GetInputIndent()
   return labelWidth;
 }
 
+void GroupCell::UpdateOutputPositions()
+{
+  UpdateYPosition();
+  if (m_output && !IsHidden())
+  {
+    wxPoint in = GetCurrentPoint();
+    auto *const configuration = (*m_configuration);
+    if (configuration->ShowCodeCells() || (m_groupType != GC_TYPE_CODE))
+      in.y += m_inputLabel->GetMaxDrop();
+    
+    m_outputRect.SetPosition(in);
+    bool first = true;
+    int drop = 0;
+    for (Cell &tmp : OnDrawList(m_output.get()))
+    {
+      if (first || tmp.BreakLineHere())
+      {
+        if (!first && tmp.HasBigSkip())
+          in.y += MC_LINE_SKIP;
+        
+        in.x = GetCurrentPoint().x + GetLineIndent(&tmp);
+        in.y += drop + tmp.GetCenterList();
+        drop = tmp.GetMaxDrop();
+      }
+      tmp.SetCurrentPoint(in);
+      in.x += tmp.GetWidth();
+      first = false;
+    }
+  }
+}
+
 void GroupCell::Draw(wxPoint const point)
 {
   Cell::Draw(point);
   Configuration *configuration = (*m_configuration);
-
+  if(NeedsRecalculation((*m_configuration)->GetDefaultFontSize()))
+    wxLogMessage(wxString::Format(_("Not recalculated: \"%s\""), ToString().utf8_str()));
   if (configuration->ShowBrackets())
     DrawBracket();
 
   if (!DrawThisCell(point))
     return;
 
-  if (!TEMPORARY_WINDOWS_PERFORMANCE_HACK)
-  {
-    if (NeedsRecalculation((*m_configuration)->GetDefaultFontSize()))
-      Recalculate();
-  }
   if (m_updateConfusableCharWarnings)
     UpdateConfusableCharWarnings();
 
@@ -792,7 +835,7 @@ void GroupCell::Draw(wxPoint const point)
 
       EditorCell *input = GetEditable();
       if (input)
-        input->Draw({point.x + GetInputIndent(), point.y});
+        input->Draw(CalculateInputPosition());
 
       if (GetPrompt())
         GetPrompt()->Draw(point);
@@ -1099,8 +1142,9 @@ wxString GroupCell::ToRTF() const
 
 wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter) const
 {
-  wxASSERT_MSG((imgCounter != NULL), _(wxT("Bug: No image counter to write to!")));
-  if (imgCounter == NULL) return wxEmptyString;
+  int myImgCounter = 0;
+  if(imgCounter == NULL)
+    imgCounter = &myImgCounter; 
   wxString str;
   switch (m_groupType)
   {
@@ -1111,7 +1155,8 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter) c
     case GC_TYPE_IMAGE:
       if (imgDir != wxEmptyString)
       {
-        auto const copy = m_output->Copy();
+        GroupCell parent(m_configuration, GC_TYPE_CODE);
+        auto const copy = m_output->Copy(&parent);
         auto *const imgCopy = dynamic_cast<ImgCell *>(copy.get());
         (*imgCounter)++;
         wxString image = filename + wxString::Format(wxT("_%d"), *imgCounter);
@@ -1290,7 +1335,8 @@ wxString GroupCell::ToTeXImage(Cell *tmp, wxString imgDir, wxString filename, in
 
   if (imgDir != wxEmptyString)
   {
-    auto const copy = tmp->Copy();
+    // TODO: Is this the right Group?
+    auto const copy = tmp->Copy(tmp->GetGroup());
     auto *const imgCopy = dynamic_cast<ImgCell *>(copy.get());
     (*imgCounter)++;
     wxString image = filename + wxString::Format(wxT("_%d"), *imgCounter);
@@ -1556,29 +1602,39 @@ void GroupCell::BreakLines()
   if(cell == NULL)
     return;
 
-  // 1st step: Break 2d objects that are wider than a line into lines
+  if(NeedsRecalculation(EditorFontSize()))
+    m_output->RecalculateList((*m_configuration)->GetMathFontSize());
+
+  // 1st step: Tell all cells to display as beautiful 2d object, if that is possible.
   if(UnBreakUpCells(cell))
   {
     m_output->ResetCellListSizesList();
     m_output->RecalculateList((*m_configuration)->GetMathFontSize());
   }
+
+  // 2nd step: Convert all objects that are wider than a line to 1D objects that (hopefully)
+  // can be broken into lines
   if(BreakUpCells(cell))
   {
     m_output->ResetCellListSizesList();
     m_output->RecalculateList((*m_configuration)->GetMathFontSize());
   }
 
-  // 2nd step: Determine a sane maximum line width
+
+  // 3rd step: Determine a sane maximum line width
   int fullWidth = (*m_configuration)->GetClientWidth();
   Configuration *configuration = (*m_configuration);
   int currentWidth = GetLineIndent(cell);
   if((cell->GetStyle() != TS_LABEL) && (cell->GetStyle() != TS_USERLABEL))
     fullWidth -= configuration->GetIndent();
 
+  // We don't want the text go exactly to the right border.
+  fullWidth -= Scale_Px(1);
+
   // Don't let the layout degenerate for small window widths
   if (fullWidth < Scale_Px(150)) fullWidth = Scale_Px(150);
 
-  // 3rd step: break the output into lines.
+  //4th step: break the output into lines.
   if (!IsHidden()) {
     bool prevBroken = false;
     for (Cell &tmp : OnDrawList(cell))
@@ -1646,18 +1702,29 @@ bool GroupCell::BreakUpCells(Cell *cell)
     showLength = 500;    
   }
 
+ int clientWidth = .8*(*m_configuration)->GetClientWidth() - (*m_configuration)->GetIndent();
+ if(clientWidth < Scale_Px(50))
+   clientWidth = Scale_Px(50);
+ 
   // Reduce the number of steps involved in layouting big equations
   if(m_cellsInGroup > showLength)
   {
-    bool lineHeightsChanged = false;
     wxLogMessage(_("Resolving to 1D layout for one cell in order to save time"));
+    return false;
+  }
+  else
+  {
+    bool lineHeightsChanged = false;
     if (!IsHidden())
       for (Cell &tmp : OnDrawList(cell))
-        lineHeightsChanged |= tmp.BreakUp();
+      {
+        if(tmp.GetWidth()<0)
+          tmp.Recalculate((*m_configuration)->GetMathFontSize());
+        if(tmp.GetWidth() > clientWidth)
+          lineHeightsChanged |= tmp.BreakUp();
+      }
     return lineHeightsChanged;
   }
-
-  return BreakUp();
 }
 
 bool GroupCell::UnBreakUpCells(Cell *cell)
@@ -1690,11 +1757,13 @@ bool GroupCell::UnBreakUpCells(Cell *cell)
  
   bool retval = false;
   for (Cell &tmp : OnDrawList(cell))
+  {
     if (tmp.IsBrokenIntoLines())
     {
       tmp.Unbreak();
       retval = true;
     }
+  }
 
   return retval;
 }
@@ -1717,6 +1786,7 @@ void GroupCell::Hide(bool hide)
   if (GetLabel())
     GetLabel()->ClearCacheList();
 
+  m_cellsAppended = true;
   ResetSize();
   GetEditable()->ResetSize();
 }
@@ -1743,12 +1813,15 @@ bool GroupCell::HideTree(std::unique_ptr<GroupCell> &&tree)
       tmp.GetLabel()->ClearCacheList();
   }
 
+  m_cellsAppended = true;
   return true;
 }
 
 std::unique_ptr<GroupCell> GroupCell::UnhideTree()
 {
   m_hiddenTree->SetHiddenTreeParent(m_hiddenTreeParent);
+  if(m_hiddenTree)
+    m_cellsAppended = true;
   return std::move(m_hiddenTree);
 }
 
@@ -1783,7 +1856,7 @@ GroupCell *GroupCell::Fold()
     return NULL;
   if (GetNext() == NULL)
     return NULL;
-  int nextgct = GetNext()->GetGroupType(); // groupType of the next cell
+  GroupType nextgct = GetNext()->GetGroupType(); // groupType of the next cell
   if ((m_groupType == nextgct) || IsLesserGCType(nextgct))
     return NULL; // if the next gc shouldn't be folded, exit
 
@@ -1808,6 +1881,7 @@ GroupCell *GroupCell::Fold()
   wxASSERT(tornOut.cellOwner);
   m_hiddenTree = static_unique_ptr_cast<GroupCell>(std::move(tornOut.cellOwner));
   m_hiddenTree->SetHiddenTreeParent(this);
+  m_cellsAppended = true;
   return this;
 }
 
@@ -1818,6 +1892,7 @@ GroupCell *GroupCell::Unfold()
   if (!IsFoldable() || !m_hiddenTree)
     return NULL;
 
+  m_cellsAppended = true;
   auto splicedIn = CellList::SpliceInAfter(this, std::move(m_hiddenTree));
   GetNext()->SetHiddenTreeParent(m_hiddenTreeParent);
   return dynamic_cast<GroupCell *>(splicedIn.lastSpliced);
@@ -1839,7 +1914,7 @@ GroupCell *GroupCell::FoldAll()
   return result;
 }
 
-// unfolds recursivly its contents
+// unfolds recursively its contents
 // if (all) then also calls it on it's m_next
 GroupCell *GroupCell::UnfoldAll()
 {
@@ -1857,7 +1932,7 @@ GroupCell *GroupCell::UnfoldAll()
   return result;
 }
 
-bool GroupCell::IsLesserGCType(int comparedTo) const
+bool GroupCell::IsLesserGCType(GroupType comparedTo) const
 {
   switch (m_groupType)
   {
@@ -1889,6 +1964,22 @@ bool GroupCell::IsLesserGCType(int comparedTo) const
   }
 }
 
+bool GroupCell::IsHeading() const
+{
+  switch (m_groupType)
+  {
+    case GC_TYPE_HEADING6:
+    case GC_TYPE_HEADING5:
+    case GC_TYPE_SUBSUBSECTION:
+    case GC_TYPE_SUBSECTION:
+    case GC_TYPE_SECTION:
+    case GC_TYPE_TITLE:
+      return true;
+  default:
+    return false;
+  }
+}
+
 void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &heading5, int &heading6, int &image) const
 {
   for (auto const &tmp : OnList(this))
