@@ -26,20 +26,21 @@
 #include "Cell.h"
 #include <wx/image.h>
 #include "Image.h"
+#include "ImgCellBase.h"
 
 #include <wx/filesys.h>
 #include <wx/fs_arc.h>
 
-class ImgCell final : public Cell
+class ImgCell final : public ImgCellBase
 {
 public:
-  ImgCell(GroupCell *parent, Configuration **config);
-  ImgCell(GroupCell *parent, Configuration **config, const wxMemoryBuffer &image, const wxString &type);
-  ImgCell(GroupCell *parent, Configuration **config, const wxString &image, std::shared_ptr<wxFileSystem> filesystem, bool remove = true);
+  ImgCell(GroupCell *group, Configuration **config);
+  ImgCell(GroupCell *group, Configuration **config, const wxMemoryBuffer &image, const wxString &type);
+  ImgCell(GroupCell *group, Configuration **config, const wxString &image, std::shared_ptr<wxFileSystem> filesystem, bool remove = true);
 
-  ImgCell(GroupCell *parent, Configuration **config, const wxBitmap &bitmap);
-  ImgCell(const ImgCell &cell);
-  std::unique_ptr<Cell> Copy() const override;
+  ImgCell(GroupCell *group, Configuration **config, const wxBitmap &bitmap);
+  ImgCell(GroupCell *group, const ImgCell &cell);
+  std::unique_ptr<Cell> Copy(GroupCell *group) const override;
   const CellTypeInfo &GetInfo() override;
   ~ImgCell() override;
 
@@ -56,10 +57,18 @@ public:
 
   void LoadImage(wxString image, bool remove = true);
 
-  //! Can this image be exported in SVG format?
-  bool CanExportSVG() const {return (m_image != NULL) && m_image->CanExportSVG();}
+  //! Set the image's resolution
+  void SetPPI(int ppi) override {m_image->SetPPI(ppi);}
+  int GetPPI() const override {return m_image->GetPPI();}
+  size_t GetOriginalWidth() const override {return m_image->GetOriginalWidth();}
+  size_t GetOriginalHeight() const override {return m_image->GetOriginalHeight();}
 
-  friend class SlideShow;
+  void ReloadImage(const wxString &image, std::shared_ptr<wxFileSystem> filesystem);
+
+  //! Can this image be exported in SVG format?
+  bool CanExportSVG() const override {return (m_image != NULL) && m_image->CanExportSVG();}
+
+  friend class AnimationCell;
 
   /*! Writes the image to a file
 
@@ -69,7 +78,7 @@ public:
 
     See also GetExtension().
    */
-  wxSize ToImageFile(wxString filename);
+  wxSize ToImageFile(wxString filename) override;
 
   /*! Removes the cached scaled image from memory
 
@@ -89,16 +98,24 @@ public:
   void DrawRectangle(bool draw) { m_drawRectangle = draw; }
 
   //! Returns the file name extension that matches the image type
-  wxString GetExtension() const
+  wxString GetExtension() const override
   { if (m_image)return m_image->GetExtension(); else return wxEmptyString; }
 
+  //! Returns the name of the file the image was originally created from
+  wxString GetOrigImageFile() const
+  { return m_origImageFile; }
+  
+  //! Sets the name of the file the image was originally created from
+  void SetOrigImageFile(wxString file)
+  { m_origImageFile = file; }
+
   //! Returns the original compressed version of the image
   wxMemoryBuffer GetCompressedImage() const { return m_image->m_compressedImage; }
 
-  double GetMaxWidth() const { return m_image ? m_image->GetMaxWidth() : -1; }
-  double GetHeightList() const { return m_image ? m_image->GetHeightList() : -1; }
-  void SetMaxWidth(double width) const { if (m_image) m_image->SetMaxWidth(width); }
-  void SetMaxHeight(double height) const { if (m_image) m_image->SetMaxHeight(height); }
+  double GetMaxWidth() const override { return m_image ? m_image->GetMaxWidth() : -1; }
+  double GetHeightList() const override { return m_image ? m_image->GetHeightList() : -1; }
+  void SetMaxWidth(double width) override { if (m_image) m_image->SetMaxWidth(width); }
+  void SetMaxHeight(double height) override { if (m_image) m_image->SetMaxHeight(height); }
 
   void Recalculate(AFontSize fontsize) override;
 
@@ -113,6 +130,7 @@ public:
   bool CanPopOut() const override { return !m_image->GnuplotSource().empty(); }
 
 private:
+  void SetConfiguration(Configuration **config) override;
   void DrawBoundingBox(wxDC &WXUNUSED(dc), bool WXUNUSED(all) = false) override;
   int GetImageBorderWidth() const override { return m_imageBorderWidth; }
 
@@ -125,7 +143,7 @@ private:
 //** Bitfield objects (1 bytes)
 //**
   void InitBitFields()
-  { // Keep the initailization order below same as the order
+  { // Keep the initialization order below same as the order
     // of bit fields in this class!
     m_drawRectangle = true;
     m_drawBoundingBox = false;
@@ -134,6 +152,8 @@ private:
   bool m_drawBoundingBox : 1 /* InitBitFields */;
 
   static int s_counter;
+  
+  wxString m_origImageFile;
 };
 
 #endif // IMGCELL_H
