@@ -34,23 +34,23 @@
 
 #define SUBSUP_DEC 3
 
-SubSupCell::SubSupCell(GroupCell *parent, Configuration **config, std::unique_ptr<Cell> &&base) :
-  Cell(parent, config),
+SubSupCell::SubSupCell(GroupCell *group, Configuration **config, std::unique_ptr<Cell> &&base) :
+  Cell(group, config),
   m_baseCell(std::move(base))
 {
   InitBitFields();
 }
 
-SubSupCell::SubSupCell(const SubSupCell &cell)
-    : SubSupCell(cell.m_group, cell.m_configuration,
-                 CopyList(cell.m_baseCell.get()))
+SubSupCell::SubSupCell(GroupCell *group, const SubSupCell &cell)
+    : SubSupCell(group, cell.m_configuration,
+                 CopyList(group, cell.m_baseCell.get()))
 {
   CopyCommonData(cell);
   m_altCopyText = cell.m_altCopyText;
-  SetIndex(CopyList(cell.m_postSubCell.get()));
-  SetExponent(CopyList(cell.m_postSupCell.get()));
-  SetPreSub(CopyList(cell.m_preSubCell.get()));
-  SetPreSup(CopyList(cell.m_preSupCell.get()));
+  SetIndex(CopyList(group,cell.m_postSubCell.get()));
+  SetExponent(CopyList(group, cell.m_postSupCell.get()));
+  SetPreSub(CopyList(group, cell.m_preSubCell.get()));
+  SetPreSup(CopyList(group, cell.m_preSupCell.get()));
 }
 
 DEFINE_CELL(SubSupCell)
@@ -435,3 +435,26 @@ wxString SubSupCell::ToXML() const
   }
   return retval;
 }
+
+wxString SubSupCell::GetDiffPart() const
+{
+  wxString s(wxT(","));
+  if (m_baseCell->IsCompound())
+    s += "(" + m_baseCell->ListToString() + ")";
+  else
+    s += m_baseCell->ListToString();
+
+  if (m_scriptCells.empty())
+  {
+    s += "[" + m_postSubCell->ListToString() + "]";
+    s += ",";
+    s += m_postSupCell->ListToString();
+  }
+  else
+  {
+    for (auto &cell : m_scriptCells)
+      s += "[" + cell->ListToString() + "]";
+    s += ",1";
+  }
+  return s;
+}
