@@ -33,9 +33,9 @@
 #include "StringUtils.h"
 #include "wx/config.h"
 
-TextCell::TextCell(GroupCell *parent, Configuration **config,
+TextCell::TextCell(GroupCell *group, Configuration **config,
                    const wxString &text, TextStyle style) :
-  Cell(parent, config)
+  Cell(group, config)
 {
   InitBitFields();
   m_textStyle = style;
@@ -74,8 +74,8 @@ TextCell::TextCell(GroupCell *parent, Configuration **config,
 // cppcheck-suppress uninitMemberVar symbolName=TextCell::m_alt
 // cppcheck-suppress uninitMemberVar symbolName=TextCell::m_altJs
 // cppcheck-suppress uninitMemberVar symbolName=TextCell::m_initialToolTip
-TextCell::TextCell(const TextCell &cell):
-    Cell(cell.m_group, cell.m_configuration),
+TextCell::TextCell(GroupCell *group, const TextCell &cell):
+    Cell(group, cell.m_configuration),
     m_text(cell.m_text),
     m_displayedText(cell.m_displayedText)
 {
@@ -221,7 +221,7 @@ void TextCell::UpdateToolTip()
     else if (m_text.Contains(S_("Illegal use of delimiter")))
       SetToolTip(&T_("Most probable cause: an operator was directly followed by a closing parenthesis."));
     else if (m_text.StartsWith(S_("find_root: function has same sign at endpoints: ")))
-      SetToolTip(&T_("find_root only works if the function the solution is searched for crosses the solution exactly once in the given range."));
+      SetToolTip(&T_("Maxima tried to find out where between two points a curve crosses the zero line. Since the curve is on the same side of the zero line in both points its algorithms fails here. Set find_root_error to false if you want it to return false instead of an error."));
     else if (m_text.StartsWith(S_("part: fell off the end.")))
       SetToolTip(&T_("part() or the [] operator was used in order to extract the nth element "
                      "of something that was less than n elements long."));
@@ -418,6 +418,9 @@ void TextCell::Draw(wxPoint point)
   if (DrawThisCell(point) &&
       !(IsHidden() || (GetHidableMultSign() && configuration->HidemultiplicationSign())))
   {
+    // An experimental go at #1655. TODO: Can we delete this line again?
+    SetPen(1);
+    
     wxDC *dc = configuration->GetDC();
     int padding = 0;
     if (GetStyle() != TS_ASCIIMATHS)
@@ -436,7 +439,6 @@ void TextCell::SetFont(AFontSize fontsize)
   Configuration *configuration = (*m_configuration);
   wxDC *dc = configuration->GetDC();
   auto style = configuration->GetStyle(m_textStyle, fontsize);
-  
   // Mark special variables that are printed as ordinary letters as being special.
   if ((!(*m_configuration)->CheckKeepPercent()) &&
       ((m_text == wxT("%e")) || (m_text == wxT("%i"))))
@@ -455,9 +457,11 @@ void TextCell::SetFont(AFontSize fontsize)
 
 bool TextCell::IsOperator() const
 {
-  if (wxString(wxT("+*/-")).Find(m_text) >= 0)
+  if (wxString(wxT("+*/-\u2212\u00B7")).Find(m_text) >= 0)
     return true;
-  if (m_text == wxT("\u2212"))
+  if(GetHidableMultSign())
+    return true;
+  if(IsHidden())
     return true;
   return false;
 }
@@ -481,7 +485,7 @@ wxString TextCell::ToString() const
       // The only way for variable or function names to contain quotes and
       // characters that clearly represent operators is that these chars
       // are quoted by a backslash: They cannot be quoted by quotation
-      // marks since maxima would'nt allow strings here.
+      // marks since maxima wouldn't allow strings here.
     {
       wxString charsNeedingQuotes("\\'\"()[]-{}^+*/&§?:;=#<>$");
       bool isOperator = true;
@@ -543,7 +547,7 @@ wxString TextCell::ToMatlab() const
     // The only way for variable or function names to contain quotes and
     // characters that clearly represent operators is that these chars
     // are quoted by a backslash: They cannot be quoted by quotation
-    // marks since maxima would'nt allow strings here.
+    // marks since maxima wouldn't allow strings here.
   {
     wxString charsNeedingQuotes("\\'\"()[]{}^+*/&§?:;=#<>$");
     bool isOperator = true;
@@ -779,7 +783,7 @@ wxString TextCell::ToTeX() const
               ((GetPrevious()->GetStyle() == TS_VARIABLE) && (GetNext()->GetStyle() == TS_VARIABLE)) &&
               // The variable name prior to this cell has no subscript
               (!(GetPrevious()->ToString().Contains(wxT('_')))) &&
-              // we will be using \mathit{} for the TeX outout.
+              // we will be using \mathit{} for the TeX output.
               ((ToString().Length() > 1) || (GetNext()->ToString().Length() > 1))
               )
         text = wxT("\\, ");
@@ -1262,11 +1266,6 @@ wxString TextCell::ToXML() const
   return wxT("<") + tag + GetXMLFlags() + wxT(">") + xmlstring + wxT("</") + tag + wxT(">");
 }
 
-wxString TextCell::GetDiffPart() const
-{
-  return wxT(",") + m_text + wxT(",1");
-}
-
 bool TextCell::IsShortNum() const
 {
   return (!GetNext()) && (m_text.Length() < 4);
