@@ -27,17 +27,21 @@
 #ifndef WXMATHML_H
 #define WXMATHML_H
 
+#include <cstddef>
 #include "precomp.h"
+#include "Configuration.h"
 #include <wx/string.h>
 #include <wx/tokenzr.h>
 
 class wxMathML
 {
  public:
-  wxMathML();
+  wxMathML(Configuration *config);
   wxString GetCmd();
  private:
   wxString m_wxMathML;
+  bool m_wxMathML_UseFile;
+  Configuration *m_configuration;
   static wxString m_maximaCMD;
 };
 
