@@ -21,9 +21,9 @@
 //  SPDX-License-Identifier: GPL-2.0+
 
 /*!\file
-  This file declares the class wxMaxima that contains most of the program's logic. 
+  This file declares the class wxMaxima that contains most of the program's logic.
 
-  The worksheet is defined in the class MathCtrl instead and 
+  The worksheet is defined in the class MathCtrl instead and
   everything surrounding it in wxMaximaFrame.
  */
 
@@ -35,7 +35,6 @@
 #include "MathParser.h"
 #include "MaximaIPC.h"
 #include "Dirstructure.h"
-
 #include <wx/socket.h>
 #include <wx/config.h>
 #include <wx/process.h>
@@ -46,13 +45,14 @@
 #include <wx/txtstrm.h>
 #include <wx/sckstrm.h>
 #include <wx/buffer.h>
+#include <wx/power.h>
 #include <memory>
 #ifdef __WXMSW__
 #include <windows.h>
 #endif
 #define DOCUMENT_VERSION_MAJOR 1
 /*! The part of the .wxmx format version number that appears after the dot.
-  
+
   - Updated to version 1.1 after user selectable animation-speeds were introduced:
     Old wxMaxima versions play them back in the default speed instead but still
     open the file.
@@ -61,14 +61,14 @@
   - Bumped to version 1.3 after sub-subsections were introduced:
     Old wxMaxima versions interpret them as subsections but still open the file.
   - Bumped to version 1.4 when we started allowing to embed .jpg images in a .wxmx
-    file. wxMaxima versions between 13.04 and 15.08 replace these images by a 
+    file. wxMaxima versions between 13.04 and 15.08 replace these images by a
     "image cannot be loaded" marker but will correctly display the rest of the file.
   - Bumped to version 1.5 when GroupCells were added an attribute that allows them to
     be used as an answer to questions.
  */
 #define DOCUMENT_VERSION_MINOR 5
 
-//! How many miliseconds should we wait between polling for stdout+cpu power?
+//! How many milliseconds should we wait between polling for stdout+cpu power?
 #define MAXIMAPOLLMSECS 2000
 
 class Maxima; // The Maxima process interface
@@ -84,7 +84,7 @@ public:
   wxMaxima(wxWindow *parent, int id, wxLocale *locale, const wxString title,
            const wxString &filename = wxEmptyString,
            const wxPoint pos = wxDefaultPosition, const wxSize size = wxDefaultSize);
-  
+
   virtual ~wxMaxima();
 
   //! Pipe maxima's output to stdout
@@ -101,16 +101,13 @@ public:
     //! The time between two auto-saves has elapsed.
             AUTO_SAVE_TIMER_ID,
     //! We look if we got new data from maxima's stdout.
-            MAXIMA_STDOUT_POLL_ID,
-            /*! We have given Maxima enough time to do the important 
-
-              now it is time to compile the list of helpfile anchors */
-            COMPILEHELPANCHORS_ID
+            MAXIMA_STDOUT_POLL_ID
   };
 
-  //! A timer that ells us that we now can do the low-prio compilation of help anchors
-  wxTimer m_compileHelpAnchorsTimer;
-  
+#ifdef wxHAS_POWER_EVENTS 
+  void OnPowerEvent(wxPowerEvent &event);
+#endif 
+
   //! Is triggered when a timer this class is responsible for requires
   void OnTimerEvent(wxTimerEvent &event);
 
@@ -120,12 +117,12 @@ public:
   void ShowTip(bool force);
 
   void SetWXMdata(const wxString &data){m_initialWorkSheetContents = data;}
-  //! Do we want to evaluate the document on statup?
+  //! Do we want to evaluate the document on startup?
   void EvalOnStartup(bool eval)
     {
       m_evalOnStartup = eval;
     }
-  
+
   //! Do we want to exit the program after evaluation?
   void ExitAfterEval(bool exitaftereval)
     {
@@ -135,12 +132,12 @@ public:
         m_logPane->SetBatchMode();
       }
     }
-  
+
   void StripLispComments(wxString &s);
 
   //! Launches the help browser on the uri passed as an argument.
   void LaunchHelpBrowser(wxString uri);
-  
+
   void SendMaxima(wxString s, bool addToHistory = false);
 
   //! Open a file
@@ -174,17 +171,12 @@ public:
   };
 
 private:
-  #ifdef HAVE_OMP_HEADER
-  omp_lock_t m_helpFileAnchorsLock;
-  #endif
   MaximaIPC m_ipc{this};
-  //! wxm data the worksheet is populated from 
+  //! wxm data the worksheet is populated from
   wxString m_initialWorkSheetContents;
   static bool m_pipeToStdout;
   static bool m_exitOnError;
   static wxString m_extraMaximaArgs;
-  //! Search for the wxMaxima help file
-  wxString SearchwxMaximaHelp();
   wxLocale *m_locale;
   //! The variable names to query for the variables pane and for internal reasons
   wxArrayString m_varNamesToQuery;
@@ -203,7 +195,7 @@ private:
   wxString m_configCommands;
   //! A RegEx that matches gnuplot errors.
   static wxRegEx m_gnuplotErrorRegex;
-  //! Clear the evaluation queue and return true if "Abort on Error" is set. 
+  //! Clear the evaluation queue and return true if "Abort on Error" is set.
   bool AbortOnError();
   //! This string allows us to detect when the string we search for has changed.
   wxString m_oldFindString;
@@ -215,12 +207,12 @@ private:
 //  static wxRegEx m_outputPromptRegEx;
   //! The number of output cells the current command has produced so far.
   int m_outputCellsFromCurrentCommand;
-  //! The maximum number of lines per command we will display 
+  //! The maximum number of lines per command we will display
   int m_maxOutputCellsPerCommand;
   /*! Double the number of consecutive unsuccessful attempts to connect to the maxima server
 
     Each prompt is deemed as but one hint for a working maxima while each crash counts twice
-    which hinders us from endlessly restarting in case maxima crashes, outputs something 
+    which hinders us from endlessly restarting in case maxima crashes, outputs something
     seemingly sensible and crashes again.
    */
   int m_unsuccessfulConnectionAttempts;
@@ -242,6 +234,20 @@ private:
   wxMemoryBuffer m_uncompletedChars;
 
 protected:
+  //! Invoke our standard wizard that generates maxima commands
+  void CommandWiz(const wxString &title,
+                  const wxString &description, const wxString &description_tooltip,
+                  const wxString &commandRule,
+                  wxString label1, wxString defaultval1, wxString tooltip1 = {},
+                  wxString label2 = {}, wxString defaultval2 = {}, wxString tooltip2 = {},
+                  wxString label3 = {}, wxString defaultval3 = {}, wxString tooltip3 = {},
+                  wxString label4 = {}, wxString defaultval4 = {}, wxString tooltip4 = {},
+                  wxString label5 = {}, wxString defaultval5 = {}, wxString tooltip5 = {},
+                          wxString label6 = {}, wxString defaultval6 = {}, wxString tooltip6 = {},
+                  wxString label7 = {}, wxString defaultval7 = {}, wxString tooltip7 = {},
+                  wxString label8 = {}, wxString defaultval8 = {}, wxString tooltip8 = {},
+                  wxString label9 = {}, wxString defaultval9 = {}, wxString tooltip9 = {}
+    );
   //! Reads a potentially unclosed XML tag and closes it
   wxString ReadPotentiallyUnclosedTag(wxStringTokenizer &lines, wxString firstLine);
 
@@ -249,16 +255,6 @@ protected:
   static wxRegEx m_xmlOpeningTagName;
   //! Looks if this opening tag is actually complete.
   static wxRegEx m_xmlOpeningTag;
-  //! Collect all keyword anchors in the help file
-  void CompileHelpFileAnchors();
-  //! Load the result from the last CompileHelpFileAnchors from the disk cache
-  bool LoadManualAnchorsFromCache();
-  //! Load the help file anchors from an wxXmlDocument
-  bool LoadManualAnchorsFromXML(wxXmlDocument xmlDocument, bool checkManualVersion = true);
-  //! Load the help file anchors from the built-in list
-  bool LoadBuiltInManualAnchors();
-  //! Save the list of help file anchors to the cache.
-  void SaveManualAnchorsToCache();
   //! The gnuplot process info
   wxProcess *m_gnuplotProcess = NULL;
   //! Info about the gnuplot process we start for querying the terminals it supports
@@ -277,19 +273,19 @@ protected:
   void OnSymbolAdd(wxCommandEvent &event);
   //! Called when the "Scroll to currently evaluated" button is pressed.
   void OnFollow(wxCommandEvent &event);
+  void OnWizardAbort(wxCommandEvent &event);
+  void OnWizardOK(wxCommandEvent &event);
+  void OnWizardInsert(wxCommandEvent &event);
 
-  /*! Get the name of the help file
-   */
-  wxString GetMaximaHelpFile();
-  //! A helper function for GetHelpFile()
-  wxString GetMaximaHelpFile2();
-
-  //! Show the help for maxima
+  //! Show the help for Maxima
   void ShowMaximaHelp(wxString = {});
 
+  //! Show the help for Maxima (without handling of anchors).
+  void ShowMaximaHelpWithoutAnchor();
+
   //! Show the help for wxMaxima
   void ShowWxMaximaHelp();
-    
+
   //! Try to determine if help is needed for maxima or wxMaxima and show this help
   void ShowHelp(const wxString &keyword);
 
@@ -304,20 +300,20 @@ protected:
 
   /*! The idle task that refreshes the gui (worksheet, menus, title line,...)
 
-    In GUIs normally all events (mouse events, key presses,...) are 
+    In GUIs normally all events (mouse events, key presses,...) are
     put into a queue and then are executed one by one. This makes
     sure that they will be processed in order - but makes a gui a
     more or less single-processor-task.
 
-    If the queue is empty (which means that the computer has caught 
+    If the queue is empty (which means that the computer has caught
     up with the incoming events) this function is called once.
 
     Moving the screen update to this function makes sure that we
-    don't do many subsequent updates slowing down the computer any 
+    don't do many subsequent updates slowing down the computer any
     further if there are still a handful of key presses to process
     lowering the average response times of the program.
 
-    The worksheet is refreshed by a timer task in case the computer 
+    The worksheet is refreshed by a timer task in case the computer
     is too busy to execute the idle task at all.
   */
   void OnIdle(wxIdleEvent &event);
@@ -331,11 +327,12 @@ protected:
    */
   bool InterpretDataFromMaxima(const wxString &newData);
   bool m_dataFromMaximaIs;
-  
+
   void MenuCommand(const wxString &cmd);           //!< Inserts command cmd into the worksheet
   void FileMenu(wxCommandEvent &event);            //!< Processes "file menu" clicks
   void MaximaMenu(wxCommandEvent &event);          //!< Processes "maxima menu" clicks
   void MatrixMenu(wxCommandEvent &event);         //!< Processes "algebra menu" clicks
+  void PropertiesMenu(wxCommandEvent &event);         //!< Processes "Variable/Function props menu" clicks
   void EquationsMenu(wxCommandEvent &event);       //!< Processes "equations menu" clicks
   void CalculusMenu(wxCommandEvent &event);        //!< event handling for menus
   void SimplifyMenu(wxCommandEvent &event);        //!< Processes "Simplify menu" clicks
@@ -363,7 +360,7 @@ protected:
 
   //! Is triggered when the textstyle drop-down box's value is changed.
   void ChangeCellStyle(wxCommandEvent &event);
-  
+
   //! Is triggered when the "Find" button in the search dialog is pressed
   void OnFind(wxFindDialogEvent &event);
 
@@ -378,7 +375,7 @@ protected:
 
   //! Is called if maxima connects to wxMaxima.
   void OnMaximaConnect();
-  
+
   //! Maxima sends or receives data, or disconnects
   void MaximaEvent(::MaximaEvent &event);
 
@@ -388,15 +385,15 @@ protected:
   /*! Add a parameter to a draw command
 
     \param cmd The parameter to  add to the draw command
-    \param dimensionsOfNewDrawCommand The number of dimensions the new draw command needs to 
+    \param dimensionsOfNewDrawCommand The number of dimensions the new draw command needs to
                                       have if we need to create one..
    */
   void AddDrawParameter(wxString cmd, int dimensionsOfNewDrawCommand = 2);
 
   /* Append something to the console. Might be Text or XML maths.
 
-    \return A pointer to the last line of Unicode text that was appended or 
-    NULL, if there is no such line (for example if the appended object is 
+    \return A pointer to the last line of Unicode text that was appended or
+    NULL, if there is no such line (for example if the appended object is
     maths instead).
    */
   TextCell *ConsoleAppend(wxString s, CellType type, const wxString &userLabel = {});        //!< append maxima output to console
@@ -449,14 +446,14 @@ protected:
   wxString ExtractFirstExpression(const wxString &entry);
 
   wxString GetDefaultEntry();
-  
+
   //! starts the server
   bool StartServer();
   /*! starts maxima (uses getCommand) or restarts it if needed
 
     Normally a restart is only needed if
       - maxima isn't currently in the process of starting up or
-      - maxima is running and has never evaluated any program 
+      - maxima is running and has never evaluated any program
         so a restart won't change anything
     \param force true means to restart maxima unconditionally.
    */
@@ -476,7 +473,7 @@ protected:
      - it discards all data until this point
      - and it prepares the worksheet for editing.
 
-     \param data The string ReadFirstPrompt() does read its data from. 
+     \param data The string ReadFirstPrompt() does read its data from.
                   After leaving this function data is empty again.
    */
   void ReadFirstPrompt(wxString &data);
@@ -509,14 +506,14 @@ protected:
 
   /*! Reads the output of wxstatusbar() commands
 
-    wxstatusbar allows the user to give and update visual feedback from long-running 
+    wxstatusbar allows the user to give and update visual feedback from long-running
     commands and makes sure this feedback is deleted once the command is finished.
    */
   void ReadStatusBar(wxString &data);
 
   /*! Reads the math cell's contents from Maxima.
-     
-     Math cells are enclosed between the tags \<mth\> and \</mth\>. 
+
+     Math cells are enclosed between the tags \<mth\> and \</mth\>.
      This function removes the from data after appending them
      to the console.
 
@@ -538,15 +535,18 @@ protected:
   /*! Reads the variable values maxima advertises to us
    */
   void ReadVariables(wxString &data);
-  
+
   /*! Reads the "add variable to watch list" tag maxima can send us
    */
   void ReadAddVariables(wxString &data);
+  void VariableActionLogexpand(const wxString &value);
   void VariableActionUserDir(const wxString &value);
   void VariableActionTempDir(const wxString &value);
+  void VariableActionDebugmode(const wxString &value);
   void VariableActionAutoconfVersion(const wxString &value);
   void VariableActionAutoconfHost(const wxString &value);
   void VariableActionMaximaInfodir(const wxString &value);
+  void VariableActionMaximaHtmldir(const wxString &value);
   void VariableActionGnuplotCommand(const wxString &value);
   void VariableActionMaximaSharedir(const wxString &value);
   void VariableActionLispName(const wxString &value);
@@ -563,6 +563,7 @@ protected:
   void VariableActionAutoplay(const wxString &value);
   void VariableActionEngineeringFormat(const wxString &value);
   void VariableActionStringdisp(const wxString &value);
+  void VariableActionOperators(const wxString &value);
   wxString m_maximaVariable_wxSubscripts;
   wxString m_maximaVariable_lmxchar;
   wxString m_maximaVariable_display2d;
@@ -596,7 +597,7 @@ protected:
     This method is called once when maxima starts. It loads wxmathml.lisp
     and sets some option variables.
 
-    \todo Set pngcairo to be the default terminal as soon as the mac platform 
+    \todo Set pngcairo to be the default terminal as soon as the mac platform
     supports it.
  */
   void SetupVariables();
@@ -616,7 +617,7 @@ protected:
     Only necessary on the mac where the same process creates loads of windows.
    */
   void BecomeLogTarget();
-  
+
   void FirstOutput();
 
   /*! Opens a content.xml file that has been extracted from a broken .wxmx file
@@ -644,14 +645,14 @@ protected:
    */
   bool SaveFile(bool forceSave = false);
 
-  //! Try to save the file before closing it - or return false 
+  //! Try to save the file before closing it - or return false
   bool SaveOnClose();
   /*! Save the project in a temp file.
 
     Returns false if a save was necessary, but not possible.
    */
   bool AutoSave();
-  
+
   int SaveDocumentP();
 
   //! Set the current working directory file I/O from maxima is relative to.
@@ -664,7 +665,25 @@ protected:
   }
 
   std::unique_ptr<Maxima> m_client;
-  wxSocketServer *m_server;
+  /*! The Right Way to delete a wxSocketServer
+
+    The destructor might delete the server before all pending server events have been
+    processed which leads to a crash.
+   */
+  struct ServerDeleter {
+  void operator()(wxSocketServer* server) const {
+    server->Close();
+    wxLogMessage(_("Closing the socket maxima could connect to!"));
+    server->Destroy();
+  }
+};
+  /*! The server maxima connects to as client
+
+     The destructor of the server causes 
+     crashes if there are still pending events.
+     Instead we need to call destroy.
+ */
+  std::unique_ptr<wxSocketServer,  ServerDeleter> m_server;
 
   wxProcess *m_process;
   //! The stdout of the maxima process
@@ -672,17 +691,15 @@ protected:
   //! The stderr of the maxima process
   wxInputStream *m_maximaStderr;
   int m_port;
-  /*! The end of maxima's current uninterpreted output, see m_currentOutput.
-   
-    If we just want to look if maxima's current output contains an ending tag
+  /*! The end of Maxima's current uninterpreted output, see m_currentOutput.
+
+    If we just want to look if Maxima's current output contains an ending tag
     this is the place we can search in fast.
 
-    wxEmptyString means that the current output isn't long enought to make 
-    creating this string worthwile.
+    wxEmptyString means that the current output isn't long enough to make
+    creating this string worthwhile.
    */
   wxString m_currentOutputEnd;
-  //! Caches the name of wxMaxima's help file.
-  wxString m_wxMaximaHelpFile;
   //! All from maxima's current output we still haven't interpreted
   wxString m_currentOutput;
   //! A marker for the start of maths
@@ -741,10 +758,8 @@ protected:
   wxString m_openFile;
   //! The directory with maxima's temp files
   wxString m_maximaTempDir;
-  //! The directory with maxima's documentation
-  wxString m_maximaDocDir;
+  wxString m_maximaHtmlDir;
   bool m_fileSaved;
-  wxString m_maximaVersion;
   wxString m_maximaArch;
   wxString m_lispVersion;
   wxString m_lispType;
@@ -761,6 +776,8 @@ protected:
   MathParser m_parser;
   bool m_maximaBusy;
 private:
+  //! The value of maxima's logexpand variable
+  wxString m_logexpand;
   //! A pointer to a method that handles a text chunk
   typedef void (wxMaxima::*ParseFunction)(wxString &s);
   WX_DECLARE_STRING_HASH_MAP(ParseFunction, ParseFunctionHash);
@@ -786,10 +803,17 @@ private:
 
     This timer is used in one-shot mode so in the unlikely case that saving needs more
     time than this timer to expire the user still got a chance to do something against
-    it between two expirys. 
+    it between two expirys.
    */
   wxTimer m_autoSaveTimer;
 
+  /* A timer that delays redraws while maxima evaluates
+
+     If we always start a redraw when maxima has nearly finished a command that slows
+     down evaluating many simple commands in a row.
+   */
+  wxTimer m_fastResponseTimer;
+
   //! Starts a single-shot of m_autoSaveTimer.
   void StartAutoSaveTimer();
 
@@ -833,10 +857,10 @@ public:
     \param file The file name
     \param evalOnStartup Do we want to execute the file automatically, but halt on error?
     \param exitAfterEval Do we want to close the window after the file has been evaluated?
-    \param wxmData A gnuzipped version of the .wxm file containing the initial worksheet contents
+    \param wxmData A .wxm file containing the initial worksheet contents
     \param wxmLen  The length of wxmData
    */
-  void NewWindow(const wxString &file = {}, bool evalOnStartup = false, bool exitAfterEval = false, unsigned char *wxmData = NULL, int wxmLen = 0);
+  void NewWindow(const wxString &file = {}, bool evalOnStartup = false, bool exitAfterEval = false, unsigned char *wxmData = NULL, size_t wxmLen = 0);
 
   static std::vector<wxMaxima *> m_topLevelWindows;
   static void DelistTopLevelWindow(wxMaxima *);
