@@ -42,6 +42,7 @@ surrounding the worksheet.
 #include <wx/notifmsg.h>
 #include <wx/wrapsizer.h>
 
+#include "ScrollingGenWizPanel.h"
 #include "Worksheet.h"
 #include "RecentDocuments.h"
 #include "Version.h"
@@ -136,6 +137,7 @@ public:
     menu_ivp_1,
     menu_ivp_2,
     menu_bvp,
+    menu_genmatrix,
     menu_gen_mat,
     menu_gen_mat_lambda,
     menu_enter_mat,
@@ -152,6 +154,24 @@ public:
     menu_matrix_exponent,
     menu_matrix_hadamard_product,
     menu_matrix_hadamard_exponent,
+    menu_matrix_loadLapack,
+    menu_matrix_dgeev_eigenvaluesOnly,
+    menu_matrix_dgeev,
+    menu_matrix_zgeev_eigenvaluesOnly,
+    menu_matrix_zgeev,
+    menu_matrix_dgeqrf,
+    menu_matrix_dgesv,
+    menu_matrix_dgesvd,
+    menu_matrix_dgesvd_valuesOnly,
+    menu_matrix_dlange_max,
+    menu_matrix_dlange_one,
+    menu_matrix_dlange_inf,
+    menu_matrix_dlange_frobenius,
+    menu_matrix_zlange_max,
+    menu_matrix_zlange_one,
+    menu_matrix_zlange_inf,
+    menu_matrix_zlange_frobenius,
+    menu_matrix_zheev,
     menu_invert_mat,
     menu_cpoly,
     menu_determinant,
@@ -183,6 +203,22 @@ public:
     menu_to_float,
     menu_to_bfloat,
     menu_to_numer,
+    menu_rat,
+    menu_rationalize,
+    menu_guess_exact_value,
+    menu_quad_qag,
+    menu_quad_qags,
+    menu_quad_qagi,
+    menu_quad_qawc,
+    menu_quad_qawf_sin,
+    menu_quad_qawf_cos,
+    menu_quad_qawo_sin,
+    menu_quad_qawo_cos,
+    menu_quad_qaws1,
+    menu_quad_qaws2,
+    menu_quad_qaws3,
+    menu_quad_qaws4,
+    menu_quad_qagp,
     menu_num_domain,
     menu_set_precision,
     menu_set_displayprecision,
@@ -206,7 +242,6 @@ public:
     menu_lbfgs,
     menu_series,
     menu_pade,
-    menu_map,
     menu_diff,
     menu_solve_de,
     menu_atvalue,
@@ -217,8 +252,6 @@ public:
     menu_example,
     menu_apropos,
     menu_product,
-    menu_make_list,
-    menu_apply,
     menu_time,
     menu_factsimp,
     menu_factcomb,
@@ -229,6 +262,10 @@ public:
     menu_math_as_1D_ASCII,
     menu_math_as_2D_ASCII,
     menu_math_as_graphics,
+    menu_logexpand_false,
+    menu_logexpand_true,
+    menu_logexpand_all,
+    menu_logexpand_super,
     menu_noAutosubscript,
     menu_defaultAutosubscript,
     menu_alwaysAutosubscript,
@@ -299,6 +336,10 @@ public:
     menu_to_fact,
     menu_to_gamma,
     menu_texform,
+    menu_debugmode,
+    menu_debugmode_off,
+    menu_debugmode_lisp,
+    menu_debugmode_all,
     button_enter,
     menu_zoom_80,
     /* Instead of menu_zoom_100 we use the standard wxID_ZOOM_100, which displays an icon in the menu (currently Unix only) */
@@ -465,6 +506,7 @@ public:
     menu_unsaved_document_27,
     menu_unsaved_document_28,
     menu_unsaved_document_29,
+    menu_construct_fraction,
     menu_insert_image,
     menu_stats_mean,
     menu_stats_median,
@@ -611,6 +653,8 @@ protected:
   StatusBar *m_statusBar;
   //! The menu bar
   MainMenuBar *m_MenuBar;
+  //! The logexpand submenu
+  wxMenu * m_logexpand_Sub;
   //! The file menu.
   wxMenu *m_FileMenu;
   //! The edit menu.
@@ -625,12 +669,14 @@ protected:
   wxMenu *m_EquationsMenu;
   //! The maxima menu.
   wxMenu *m_MaximaMenu;
-  //! The algebra menu.
-  wxMenu *m_Algebra_Menu;
+  //! The matrix menu.
+  wxMenu *m_matrix_menu;
   //! The simplify menu
   wxMenu *m_SimplifyMenu;
   //! The factorials and gamma submenu
   wxMenu *m_Simplify_Gamma_Sub;
+  //! Contains the menu for the debugger trigger settingxc
+  wxMenu *m_debugTypeMenu;
   //! The trigonometric submenu
   wxMenu *m_Simplify_Trig_Sub;
   //! The complex submenu
@@ -718,6 +764,7 @@ public:
   void RightStatusText(const wxString &text, bool saveInLog = true)
     {m_newRightStatusText = true; m_rightStatusText = text; if(saveInLog)wxLogMessage(text);}
 protected:
+  ScrollingGenWizPanel *m_wizard = NULL;
   //! Are we inside a 2d or 3d draw command?
   long m_drawDimensions_last;
   //! Do we have new text to output in the Right half of the Status Bar?
@@ -772,6 +819,8 @@ private:
   wxPanel *CreateSymbolsPane();
 
 protected:
+  bool m_historyVisible;
+  bool m_xmlMonitorVisible;
   SymbolsPane *m_symbolsPane;
   //! The current length of the evaluation queue of commands we still need to send to maxima
   int m_EvaluationQueueLength;
